import numpy as np
import cv2
import popUpAlgoFollow

def getBackground(videoPath, hyperparameters):

  cap = cv2.VideoCapture(videoPath)
  max_l      = int(cap.get(7))

  backCalculationStep = hyperparameters["backCalculationStep"]
  if "firstFrame" in hyperparameters:
    firstFrame = hyperparameters["firstFrame"]
  else:
    firstFrame = 1
  if "lastFrame" in hyperparameters:
    lastFrame  = hyperparameters["lastFrame"]
  else:
    lastFrame  = max_l - 10
  debugExtractBack    = hyperparameters["debugExtractBack"]
  
  if backCalculationStep == -1:
    backCalculationStep = int((lastFrame - firstFrame) / hyperparameters["nbImagesForBackgroundCalculation"])
    if backCalculationStep <= 1:
      backCalculationStep = 1
  
  ret, back = cap.read()
  back = cv2.cvtColor(back, cv2.COLOR_BGR2GRAY)
  for k in range(firstFrame,lastFrame):
    if (k % backCalculationStep == 0):
      cap.set(1, k)
      ret, frame = cap.read()
      if debugExtractBack:
        print(k)
      if ret:
        frame = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        if hyperparameters["extractBackWhiteBackground"]:
          back = cv2.max(frame, back)
        else:
          back = cv2.min(frame, back)
      else:
        print("couldn't use the frame", k, "for the background extraction")

  if (debugExtractBack):
    cv2.imshow('Background extracted', back)
    if hyperparameters["exitAfterBackgroundExtraction"]:
      cv2.waitKey(3000)
    else:
      cv2.waitKey(0)
    cv2.destroyAllWindows()
    
  cap.release()
  
  print("Background Extracted")
  if hyperparameters["popUpAlgoFollow"]:
    popUpAlgoFollow.prepend("Background Extracted")
  
  return back



import cv2
import math
import os

from headEmbededTailTracking import headEmbededTailTracking
from centerOfMassTailTracking import centerOfMassTailTracking
from tailTrackingExtremityDetect import tailTrackingExtremityDetect
from tailTrackingBlobDescent import tailTrackingBlobDescent
from getTailTipManual import findHeadPositionByUserInput
from headEmbededTailTrackingTeresaNicolson import headEmbededTailTrackingTeresaNicolson

def tailTracking(animalId, i, firstFrame, heading, videoPath, x, y, headPosition, frame, hyperparameters, thresh1, nbTailPoints, threshForBlackFrames, thetaDiffAccept, output, lastFirstTheta, maxDepth, tailTip):
  
  if (hyperparameters["headEmbeded"] == 1):
    # through the "head embeded" method, either through "segment descent" or "center of mass descent"
    
    if hyperparameters["headEmbededTeresaNicolson"] == 1:
      oppHeading = (heading + math.pi) % (2 * math.pi)
      outputI = headEmbededTailTrackingTeresaNicolson(headPosition,nbTailPoints,i,thresh1,frame,hyperparameters,oppHeading,maxDepth,tailTip,threshForBlackFrames)
    else:
      oppHeading = (heading + math.pi) % (2 * math.pi) # INSERTED FOR THE REFACTORING
      if hyperparameters["centerOfMassTailTracking"] == 0:
        outputI = headEmbededTailTracking(headPosition,nbTailPoints,i,thresh1,frame,hyperparameters,oppHeading,maxDepth,tailTip)
      else:
        outputI = centerOfMassTailTracking(headPosition,nbTailPoints,i,thresh1,frame,hyperparameters,oppHeading,maxDepth)
    
    if len(outputI[0]) == len(output[animalId, i-firstFrame]):
      output[animalId, i-firstFrame] = outputI

  else:
    if hyperparameters["freeSwimmingTailTrackingMethod"] == "tailExtremityDetect":
      # through the tail extremity descent method (original C++ method)
      outputI = tailTrackingExtremityDetect(headPosition,nbTailPoints,i,thresh1,frame,hyperparameters["debugTrackingPtExtreme"],heading,hyperparameters)
      output[animalId, i-firstFrame] = outputI
    elif hyperparameters["freeSwimmingTailTrackingMethod"] == "blobDescent":
      # through the "blob descent" method
      outputI = tailTrackingBlobDescent(headPosition,nbTailPoints,i,x,y,thresh1,frame,lastFirstTheta,hyperparameters["debugTrackingPtExtreme"],thetaDiffAccept,hyperparameters)
      output[animalId, i-firstFrame] = outputI
    else: # hyperparameters["freeSwimmingTailTrackingMethod"] == "none"
      # only tracking the head, not the tail
      output[animalId, i-firstFrame][0][0] = headPosition[0]
      output[animalId, i-firstFrame][0][1] = headPosition[1]
      
  return [output, maxDepth, tailTip, headPosition]



import tkinter as tk
from tkinter import font  as tkfont
from tkinter import filedialog
from tkinter import ttk
from tkinter import *
import re
import os
import json
import subprocess
import matplotlib
matplotlib.use("TkAgg")
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.figure import Figure
import math
import scipy.io as sio
from readValidationVideo import readValidationVideo

from mainZZ import mainZZ
from getTailExtremityFirstFrame import getTailExtremityFirstFrame
import popUpAlgoFollow

import configFilePrepareFunctions
import GUI_InitialFunctions
import configFileZebrafishFunctions
import adjustParameterInsideAlgoFunctions
import dataAnalysisGUIFunctions
import troubleshootingFunction
from GUI_InitialClasses import FullScreenApp, StartPage, SeveralVideos, VideoToAnalyze, FolderToAnalyze, TailExtremityHE, ConfigFilePromp, Patience, ZZoutro, ResultsVisualization, ViewParameters
from configFilePrepare import ChooseVideoToCreateConfigFileFor, ChooseGeneralExperiment, WellOrganisation, CircularWells, NumberOfAnimals, IdentifyHeadCenter, IdentifyBodyExtremity, FinishConfig, ChooseCircularWellsLeft, ChooseCircularWellsRight, GoToAdvanceSettings
from configFileZebrafish import HeadEmbeded
from adjustParameterInsideAlgo import AdujstParamInsideAlgo, AdujstParamInsideAlgoFreelySwim
from dataAnalysisGUI import CreateExperimentOrganizationExcel, ChooseExperimentOrganizationExcel, ChooseDataAnalysisMethod, PopulationComparison, BoutClustering, AnalysisOutputFolderPopulation, AnalysisOutputFolderClustering
from troubleshooting import ChooseVideoToTroubleshootSplitVideo, VideoToTroubleshootSplitVideo


LARGE_FONT= ("Verdana", 12)

def getCurrentResultFolder():
  return currentResultFolder

class SampleApp(tk.Tk):

    def __init__(self, *args, **kwargs):
        tk.Tk.__init__(self, *args, **kwargs)
        self.title("ZebraZoom")
        self.currentResultFolder = "abc"
        self.homeDirectory = os.path.dirname(os.path.realpath(__file__))
        
        self.configFile = {}
        self.videoToCreateConfigFileFor = ''
        self.wellLeftBorderX = 0
        self.wellLeftBorderY = 0
        self.headCenterX = 0
        self.headCenterY = 0
        self.organism = ''
        
        self.numWell = 0
        self.numPoiss = 0
        self.numMouv = 0
        self.visualization = 0
        self.justEnteredViewParameter = 0
        self.dataRef = {}
        self.title_font = tkfont.Font(family='Helvetica', size=18, weight="bold", slant="italic")   
        FullScreenApp(self)
        container = tk.Frame(self)
        container.pack(side="top", fill="both", expand=True)
        container.grid_rowconfigure(0, weight=1)
        container.grid_columnconfigure(0, weight=1) 
        self.container = container
        self.frames = {}
        for F in (StartPage, SeveralVideos, VideoToAnalyze, ChooseVideoToCreateConfigFileFor, ChooseGeneralExperiment, WellOrganisation, CircularWells, NumberOfAnimals, IdentifyHeadCenter, IdentifyBodyExtremity, ChooseCircularWellsLeft, ChooseCircularWellsRight, FinishConfig, FolderToAnalyze, TailExtremityHE, ConfigFilePromp, Patience, ZZoutro, ResultsVisualization, ViewParameters, HeadEmbeded, AdujstParamInsideAlgo, AdujstParamInsideAlgoFreelySwim, GoToAdvanceSettings, CreateExperimentOrganizationExcel, ChooseExperimentOrganizationExcel, ChooseDataAnalysisMethod, PopulationComparison, BoutClustering, AnalysisOutputFolderPopulation, AnalysisOutputFolderClustering, ChooseVideoToTroubleshootSplitVideo, VideoToTroubleshootSplitVideo):
            page_name = F.__name__
            frame = F(parent=container, controller=self)
            self.frames[page_name] = frame
            frame.grid(row=0, column=0, sticky="nsew")
        self.show_frame("StartPage")

    def show_frame(self, page_name):
        '''Show a frame for the given page name'''
        frame = self.frames[page_name]
        frame.tkraise()
        
    def chooseVideoToAnalyze(self, justExtractParams, noValidationVideo):
        GUI_InitialFunctions.chooseVideoToAnalyze(self, justExtractParams, noValidationVideo)

    def chooseFolderToAnalyze(self, justExtractParams, noValidationVideo):
        GUI_InitialFunctions.chooseFolderToAnalyze(self, justExtractParams, noValidationVideo)
        
    def chooseFolderForTailExtremityHE(self):
        GUI_InitialFunctions.chooseFolderForTailExtremityHE(self)
    
    def chooseConfigFile(self):
        GUI_InitialFunctions.chooseConfigFile(self)
        
    def launchZebraZoom(self):
        GUI_InitialFunctions.launchZebraZoom(self)
        
    def showResultsVisualization(self):
        self.frames['ResultsVisualization'].destroy()
        frame = ResultsVisualization(parent=self.container,controller=self)
        frame.grid(row=0, column=0, sticky="nsew")
        self.frames['ResultsVisualization'] = frame
        self.show_frame("ResultsVisualization")

    def showValidationVideo(self, numWell, zoom, deb):
        GUI_InitialFunctions.showValidationVideo(self, numWell, zoom, deb)

    def printSomeResults(self, numWell, numPoiss, numMouv, changeVisualization=False):
        if changeVisualization:
            self.visualization = int(self.visualization + 1) % 3
        
        self.numWell  = int(numWell)
        self.numPoiss = int(numPoiss)
        self.numMouv  = int(numMouv)
        if self.numWell < 0:
            self.numWell = 0
        if self.numPoiss < 0:
            self.numPoiss = 0
        if self.numMouv < 0:
            self.numMouv = 0
        self.frames['ViewParameters'].destroy()
        frame = ViewParameters(parent=self.container,controller=self)
        frame.grid(row=0, column=0, sticky="nsew")
        self.frames['ViewParameters']=frame
        self.show_frame("ViewParameters")
        
    def exploreResultFolder(self, currentResultFolder):
        GUI_InitialFunctions.exploreResultFolder(self, currentResultFolder)
        
    def printNextResults(self, numWell, numPoiss, numMouv, nbWells, nbPoiss, nbMouv):
        GUI_InitialFunctions.printNextResults(self, numWell, numPoiss, numMouv, nbWells, nbPoiss, nbMouv)

    def printPreviousResults(self, numWell, numPoiss, numMouv, nbWells, nbPoiss, nbMouv):
        GUI_InitialFunctions.printPreviousResults(self, numWell, numPoiss, numMouv, nbWells, nbPoiss, nbMouv)

    def flagMove(self, numWell, numPoiss, numMouv):
        GUI_InitialFunctions.flagMove(self, numWell, numPoiss, numMouv)

    def saveSuperStruct(self, numWell, numPoiss, numMouv):
        GUI_InitialFunctions.saveSuperStruct(self, numWell, numPoiss, numMouv)
        
    def openConfigurationFileFolder(self, homeDirectory):
        GUI_InitialFunctions.openConfigurationFileFolder(self, homeDirectory)
        
    def openZZOutputFolder(self, homeDirectory):
        GUI_InitialFunctions.openZZOutputFolder(self, homeDirectory)
        
    # Config File preparation functions
    
    def chooseVideoToCreateConfigFileFor(self, controller):
        configFilePrepareFunctions.chooseVideoToCreateConfigFileFor(self, controller)
    
    def chooseGeneralExperiment(self, controller, freeZebra, headEmbZebra, drosophilia, rodent, other):
        configFilePrepareFunctions.chooseGeneralExperiment(self, controller, freeZebra, headEmbZebra, drosophilia, rodent, other)
        
    def wellOrganisation(self, controller, circular, roi, other):
        configFilePrepareFunctions.wellOrganisation(self, controller, circular, roi, other)
        
    def circularWells(self, controller, nbwells, nbRowsOfWells, nbWellsPerRows):
        configFilePrepareFunctions.circularWells(self, controller, nbwells, nbRowsOfWells, nbWellsPerRows)
        
    def finishConfig(self, controller, configFileNameToSave):
        configFilePrepareFunctions.finishConfig(self, controller, configFileNameToSave)
        
    def chooseCircularWellsLeft(self, controller):
        configFilePrepareFunctions.chooseCircularWellsLeft(self, controller)

    def chooseCircularWellsRight(self, controller):
        configFilePrepareFunctions.chooseCircularWellsRight(self, controller)

    def numberOfAnimals(self, controller, nbanimals, yes, noo):
      configFilePrepareFunctions.numberOfAnimals(self, controller, nbanimals, yes, noo)
        
    def chooseHeadCenter(self, controller):
        configFilePrepareFunctions.chooseHeadCenter(self, controller)

    def chooseBodyExtremity(self, controller):
        configFilePrepareFunctions.chooseBodyExtremity(self, controller)
    
    def chooseBeginningAndEndOfVideo(self, controller):
        configFilePrepareFunctions.chooseBeginningAndEndOfVideo(self, controller)

    def headEmbededGUI(self, controller, blackBack, whiteBack, noBoutDetect, boutDetection, optionExtendedDescentSearchOption, 
optionBackgroundExtractionOption):
        configFileZebrafishFunctions.headEmbededGUI(self, controller, blackBack, whiteBack, noBoutDetect, boutDetection, optionExtendedDescentSearchOption, optionBackgroundExtractionOption)

    def detectBouts(self, controller, wellNumber, firstFrameParamAdjust, adjustOnWholeVideo):
      adjustParameterInsideAlgoFunctions.detectBouts(self, controller, wellNumber, firstFrameParamAdjust, adjustOnWholeVideo)
      
    def adjustHeadEmbededTracking(self, controller, wellNumber, firstFrameParamAdjust, adjustOnWholeVideo):
      adjustParameterInsideAlgoFunctions.adjustHeadEmbededTracking(self, controller, wellNumber, firstFrameParamAdjust, adjustOnWholeVideo)
    
    def adjustFreelySwimTracking(self, controller, wellNumber, firstFrameParamAdjust, adjustOnWholeVideo):
      adjustParameterInsideAlgoFunctions.adjustFreelySwimTracking(self, controller, wellNumber, firstFrameParamAdjust, adjustOnWholeVideo)

    def calculateBackground(self, controller, nbImagesForBackgroundCalculation):
      adjustParameterInsideAlgoFunctions.calculateBackground(self, controller, nbImagesForBackgroundCalculation)
      
    def calculateBackgroundFreelySwim(self, controller, nbImagesForBackgroundCalculation):
      adjustParameterInsideAlgoFunctions.calculateBackgroundFreelySwim(self, controller, nbImagesForBackgroundCalculation)
    
    def chooseVideoToCreateConfigFileFor(self, controller, reloadConfigFile):
      configFilePrepareFunctions.chooseVideoToCreateConfigFileFor(self, controller, reloadConfigFile)

    def goToAdvanceSettings(self, controller, yes, no):
      configFilePrepareFunctions.goToAdvanceSettings(self, controller, yes, no)
      
    def openExperimentOrganizationExcelFolder(self, homeDirectory):
      dataAnalysisGUIFunctions.openExperimentOrganizationExcelFolder(self, homeDirectory)
    
    def chooseExperimentOrganizationExcel(self, controller):
      dataAnalysisGUIFunctions.chooseExperimentOrganizationExcel(self, controller)
  
    def populationComparison(self, controller, BoutDuration, TotalDistance, Speed, NumberOfOscillations, meanTBF, maxAmplitude):
      dataAnalysisGUIFunctions.populationComparison(self, controller, BoutDuration, TotalDistance, Speed, NumberOfOscillations, meanTBF, maxAmplitude)
      
    def boutClustering(self, controller, nbClustersToFind, FreelySwimming, HeadEmbeded):
      dataAnalysisGUIFunctions.boutClustering(self, controller, nbClustersToFind, FreelySwimming, HeadEmbeded)
    
    def openPopulationAnalysisFolder(self, homeDirectory):
      dataAnalysisGUIFunctions.openPopulationAnalysisFolder(self, homeDirectory)
    
    def openClusteringAnalysisFolder(self, homeDirectory):
      dataAnalysisGUIFunctions.openClusteringAnalysisFolder(self, homeDirectory)    
    
    def chooseVideoToTroubleshootSplitVideo(self, controller):
      troubleshootingFunction.chooseVideoToTroubleshootSplitVideo(self, controller)



import cv2

from headTrackingTakeHeadClosestToWellCenter import headTrackingTakeHeadClosestToWellCenter
from calculateHeading import calculateHeading
from multipleAnimalsHeadTracking import multipleAnimalsHeadTracking
from multipleAnimalsHeadTrackingAdvance import multipleAnimalsHeadTrackingAdvance
# from postProcessingMultiAnimalTrajectories import postProcessingMultiAnimalTrajectories

def headTrackingHeadingCalculation(hyperparameters, firstFrame, i, blur, thresh1, thresh2, gray, erodeSize, frame_width, frame_height, outputHeading, output, headPosition, lengthX):

  xHB_TN = 0
  heading = 0
  x = 0
  y = 0
  lastFirstTheta = 0
  
  if hyperparameters["nbAnimalsPerWell"] > 1:
    
    if hyperparameters["multipleAnimalTrackingAdvanceAlgorithm"] == 0:
      [outputHeading, output] = multipleAnimalsHeadTracking(outputHeading, output, hyperparameters, gray, i, firstFrame, thresh1, thresh2)
    else:
      [outputHeading, output] = multipleAnimalsHeadTrackingAdvance(outputHeading, output, hyperparameters, gray, i, firstFrame, thresh1, thresh2, lengthX)
    
    heading = 0
    headPosition = []
    x = 0
    y = 0
    lastFirstTheta = 0
    
  else:
    
    if (hyperparameters["headEmbeded"] == 1 and i == firstFrame) or (hyperparameters["headEmbeded"] == 0) or (hyperparameters["headEmbededTeresaNicolson"] == 1):
      
      if hyperparameters["findHeadPositionByUserInput"] == 0:
        # Finds head position for frame i
        takeTheHeadClosestToTheCenter = hyperparameters["takeTheHeadClosestToTheCenter"]
        if takeTheHeadClosestToTheCenter == 0:
          (minVal, maxVal, headPosition, maxLoc) = cv2.minMaxLoc(blur)
        else:
          headPosition = headTrackingTakeHeadClosestToWellCenter(thresh1, thresh2, blur, erodeSize, hyperparameters["minArea"], hyperparameters["maxArea"], frame_width, frame_height)
        
        x = headPosition[0]
        y = headPosition[1]
        
        if (hyperparameters["headEmbededTeresaNicolson"] == 1) and (i == firstFrame):
          xHB_TN = x
        
        if (hyperparameters["headEmbededTeresaNicolson"] == 1):
          headPosition = [xHB_TN + 100, y]
        
        # Calculate heading for frame i
        [heading, lastFirstTheta] = calculateHeading(x, y, i, thresh1, thresh2, takeTheHeadClosestToTheCenter, hyperparameters)
        
        if (hyperparameters["headEmbededTeresaNicolson"] == 1):
          heading = 0
          lastFirstTheta = 0
        
        output[0, i-firstFrame][0][0] = headPosition[0]
        output[0, i-firstFrame][0][1] = headPosition[1]
        outputHeading[0, i-firstFrame] = heading
        
      else:
        
        # This is for the head-embedeed: at this point, this is set again in the tail tracking (the heading is set in the tail tracking as well)
        output[0, i-firstFrame][0][0] = headPosition[0]
        output[0, i-firstFrame][0][1] = headPosition[1]
        
    else:
      # If head embeded, heading and head position stay the same for all frames
      outputHeading[0, i-firstFrame] = outputHeading[0, i-firstFrame-1]
      output[0, i-firstFrame]        = output[0, i-firstFrame-1]
    
  return [outputHeading, output, heading, headPosition, x, y, lastFirstTheta]



import h5py
import numpy as np
import cv2
import math
import json
import sys
import matplotlib.pyplot as plt
from scipy import interpolate
from getForegroundImage import getForegroundImage
from headEmbededFrame import headEmbededFrame
from scipy.interpolate import UnivariateSpline
from numpy import linspace

def Rotate(boundary, aaa, bbb, angle, dst):

  gauche = 0
  haut   = 0

  x1 = aaa + 100*math.cos(angle)
  y1 = bbb + 100*math.sin(angle)
  x2 = aaa + 100*math.cos(angle + 3.14159265)
  y2 = bbb + 100*math.sin(angle + 3.14159265)
  x = 0
  y = 0
  r = 0
  Yoo = [x1 + gauche,y1 + haut]
  Yaa = [x2 + gauche,y2 + haut]

  dist1 = 0
  min_dist1 = 1000000
  dist2 = 0
  min_dist2 = 1000000
  theta = 0
  alpha = 0
  alpha_aux = 0
  final_angle = 0
  for i in range(0, len(boundary)):
    Pt = boundary[i][0]
    dist1 = (Pt[0] - x1)*(Pt[0] - x1) + (Pt[1] - y1)*(Pt[1] - y1)
    dist2 = (Pt[0] - x2)*(Pt[0] - x2) + (Pt[1] - y2)*(Pt[1] - y2)
    if (dist1<min_dist1):
      min_dist1 = dist1
    if (dist2<min_dist2):
      min_dist2 = dist2

  if (min_dist1<min_dist2):
    theta = angle
  else:
    theta = angle + 3.14159265

  theta = (3.14159265/2) - theta

  for i in range(0, len(boundary)):
    Pt = boundary[i][0]
    x = Pt[0]
    y = Pt[1]
    x = x - aaa
    y = y - bbb
    r = math.sqrt(x*x + y*y)
    if (x>0):
      alpha = math.atan(y/x)
    if (x<0):
      x = -x
      alpha_aux = math.atan(y/x)
      alpha = 3.14159265 - alpha_aux
    if (x == 0):
      if (y>0):
        alpha = 3.14159265/2
      else:
        alpha = -3.14159265/2

    final_angle = theta + alpha
    x = r*math.cos(final_angle)
    y = r*math.sin(final_angle)
    Pt[0] = x + aaa
    Pt[1] = y + bbb + 200
    
    boundary[i] = Pt
    
  return boundary



import tkinter as tk
from tkinter import font  as tkfont
from tkinter import filedialog
from tkinter import ttk
from tkinter import *
from vars import getGlobalVariables
globalVariables = getGlobalVariables()


class ChooseVideoToCreateConfigFileFor(tk.Frame):

  def __init__(self, parent, controller):
  
    tk.Frame.__init__(self, parent)
    self.controller = controller
    label = tk.Label(self, text="Prepare Config File", font=controller.title_font)
    label.pack(side="top", fill="x", pady=10)
    
    reloadConfigFile = IntVar()
    Checkbutton(self, text="Click here to start from a configuration file previously created (instead of from scratch).", variable=reloadConfigFile).pack()
    
    tk.Button(self, text="Select the video you want to create a configuration file for.", command=lambda: controller.chooseVideoToCreateConfigFileFor(controller, reloadConfigFile.get())).pack()
    
    tk.Label(self, text='').pack()
    tk.Label(self, text="(you will be able to use the configuration file you create for all videos that are similar to that video)").pack()
    tk.Label(self, text="").pack()
    
    tk.Button(self, text="Go to the start page", command=lambda: controller.show_frame("StartPage")).pack()
    
    tk.Label(self, text='').pack()
    tk.Label(self, text='Warning: This procedure to create configuration files is incomplete.').pack()
    tk.Label(self, text='You may not succeed at making a good configuration file to analyze your video.').pack()
    tk.Label(self, text="If you don't manage to get a good configuration file that fits your needs, email us at info@zebrazoom.org.").pack()


class ChooseGeneralExperiment(tk.Frame):

  def __init__(self, parent, controller):
  
    tk.Frame.__init__(self, parent)
    self.controller = controller
    label = tk.Label(self, text="Prepare Config File", font=controller.title_font)
    label.pack(side="top", fill="x", pady=10)
    
    tk.Label(self, text="Choose only one of the options below:", font=("Helvetica", 12)).pack(side="top", fill="x", pady=10)
    
    freeZebra = IntVar()
    Checkbutton(self, text="Track heads and tails of freely swimming zebrafish.", variable=freeZebra).pack()
    tk.Label(self, text="The zebrafish must be in one well of any shape or in several circular wells. Several zebrafish can be tracked but with only one zebrafish per well.").pack()
    tk.Label(self, text="").pack()
    
    headEmbZebra = IntVar()
    Checkbutton(self, text="Track tail of one head embedded zebrafish larva.", variable=headEmbZebra).pack()
    tk.Label(self, text="").pack()
    
    drosophilia = IntVar()
    # Checkbutton(self, text="Drosophilia", variable=drosophilia).pack()
    rodent = IntVar()
    # Checkbutton(self, text="Mouse / Rodent", variable=rodent).pack()
    other = IntVar()
    Checkbutton(self, text="Track centers of mass of any kind of animal.", variable=other).pack()
    tk.Label(self, text='Several animals can be tracked at once. The animals must be "darker" than the background and the background must be still.').pack()
    tk.Label(self, text="").pack()
    
    tk.Button(self, text="Next", command=lambda: controller.chooseGeneralExperiment(controller, freeZebra.get(), headEmbZebra.get(), drosophilia.get(), rodent.get(), other.get())).pack()
    
    tk.Button(self, text="Go to the start page", command=lambda: controller.show_frame("StartPage")).pack()


class WellOrganisation(tk.Frame):

  def __init__(self, parent, controller):
  
    tk.Frame.__init__(self, parent)
    self.controller = controller
    label = tk.Label(self, text="Prepare Config File", font=controller.title_font)
    label.pack(side="top", fill="x", pady=10)
    
    tk.Label(self, text="Choose only one of the options below:", font=("Helvetica", 12)).pack(side="top", fill="x", pady=10)
    
    circular = IntVar()
    Checkbutton(self, text="Circular Wells", variable=circular).pack()
    # rectangular = IntVar()
    # Checkbutton(self, text="Rectangular Wells", variable=rectangular).pack()
    roi = IntVar()
    Checkbutton(self, text="Choose Region of Interest", variable=roi).pack()
    other = IntVar()
    Checkbutton(self, text="Whole video", variable=other).pack()
    
    
    tk.Button(self, text="Next", command=lambda: controller.wellOrganisation(controller, circular.get(), roi.get(), other.get())).pack()
    
    tk.Button(self, text="Go to the start page", command=lambda: controller.show_frame("StartPage")).pack()


class CircularWells(tk.Frame):

  def __init__(self, parent, controller):
  
    tk.Frame.__init__(self, parent)
    self.controller = controller
    label = tk.Label(self, text="Prepare Config File", font=controller.title_font)
    label.pack(side="top", fill="x", pady=10)
    
    tk.Label(self, text="How many wells are there in your video?", font=("Helvetica", 10)).pack(side="top", fill="x", pady=10)
    nbwells = tk.Entry(self)
    nbwells.pack()
    
    tk.Label(self, text="How many rows of wells are there in your video? (leave blank for default of 1)", font=("Helvetica", 10)).pack(side="top", fill="x", pady=10)
    nbRowsOfWells = tk.Entry(self)
    nbRowsOfWells.pack()
    
    tk.Label(self, text="How many wells are there per row on your video? (leave blank for default of 4)", font=("Helvetica", 10)).pack(side="top", fill="x", pady=10)
    nbWellsPerRows = tk.Entry(self)
    nbWellsPerRows.pack()
    
    tk.Button(self, text="Next", command=lambda: controller.circularWells(controller, nbwells.get(), nbRowsOfWells.get(), nbWellsPerRows.get())).pack()
    
    tk.Button(self, text="Go to the start page", command=lambda: controller.show_frame("StartPage")).pack()


class ChooseCircularWellsLeft(tk.Frame):

  def __init__(self, parent, controller):
  
    tk.Frame.__init__(self, parent)
    self.controller = controller
    label = tk.Label(self, text="Prepare Config File", font=controller.title_font)
    label.pack(side="top", fill="x", pady=10)
    
    tk.Button(self, text="Click on the left inner border of the top left well", command=lambda: controller.chooseCircularWellsLeft(controller)).pack()
    
    tk.Label(self, text="Example:", font=("Helvetica", 10)).pack(side="top", fill="x", pady=10)
    
    canvas = Canvas(self, width=2500, height=2000)
    canvas.pack()
    img = PhotoImage(file="./code/GUI/leftborder.png")
    self.img = img
    canvas.create_image(20, 20, anchor=NW, image=img)
    
    tk.Button(self, text="Go to the start page", command=lambda: controller.show_frame("StartPage")).pack()


class ChooseCircularWellsRight(tk.Frame):

  def __init__(self, parent, controller):
  
    tk.Frame.__init__(self, parent)
    self.controller = controller
    label = tk.Label(self, text="Prepare Config File", font=controller.title_font)
    label.pack(side="top", fill="x", pady=10)
    
    tk.Button(self, text="Click on the right inner border of the top left well", command=lambda: controller.chooseCircularWellsRight(controller)).pack()
    
    tk.Label(self, text="Example:", font=("Helvetica", 10)).pack(side="top", fill="x", pady=10)
    
    canvas = Canvas(self, width=2500, height=2000)
    canvas.pack()
    img = PhotoImage(file="./code/GUI/rightborder.png")
    self.img = img
    canvas.create_image(20, 20, anchor=NW, image=img)
    
    tk.Button(self, text="Go to the start page", command=lambda: controller.show_frame("StartPage")).pack()


class NumberOfAnimals(tk.Frame):

  def __init__(self, parent, controller):
  
    tk.Frame.__init__(self, parent)
    self.controller = controller
    label = tk.Label(self, text="Prepare Config File", font=controller.title_font)
    label.pack(side="top", fill="x", pady=10)
    
    tk.Label(self, text="What's the total number of animals in your video?", font=("Helvetica", 10)).pack(side="top", fill="x", pady=10)
    
    nbanimals = tk.Entry(self)
    nbanimals.pack()
    
    tk.Label(self, text="", font=("Helvetica", 10)).pack(side="top", fill="x", pady=10)
    tk.Label(self, text="Are all of those animals ALWAYS visible throughout the video?", font=("Helvetica", 10)).pack(side="top", fill="x", pady=10)
    
    yes = IntVar()
    Checkbutton(self, text="Yes", variable=yes).pack()
    noo = IntVar()
    Checkbutton(self, text="No", variable=noo).pack()
    
    tk.Label(self, text="", font=("Helvetica", 10)).pack(side="top", fill="x", pady=10)
    tk.Button(self, text="Next", command=lambda: controller.numberOfAnimals(controller, nbanimals.get(), yes.get(), noo.get())).pack()
    tk.Label(self, text="", font=("Helvetica", 10)).pack(side="top", fill="x", pady=10)
    tk.Button(self, text="Go to the start page", command=lambda: controller.show_frame("StartPage")).pack()


class IdentifyHeadCenter(tk.Frame):

  def __init__(self, parent, controller):
  
    tk.Frame.__init__(self, parent)
    self.controller = controller
    label = tk.Label(self, text="Prepare Config File", font=controller.title_font)
    label.pack(side="top", fill="x", pady=10)
    
    tk.Button(self, text="Click on the center of the head of a zebrafish", command=lambda: controller.chooseHeadCenter(controller)).pack()
    
    tk.Label(self, text="Example:", font=("Helvetica", 10)).pack(side="top", fill="x", pady=10)
    
    canvas = Canvas(self, width=2500, height=2000)
    canvas.pack()
    img = PhotoImage(file="./code/GUI/blobCenter.png")
    self.img = img
    canvas.create_image(20, 20, anchor=NW, image=img)
    
    tk.Button(self, text="Go to the start page", command=lambda: controller.show_frame("StartPage")).pack()

    
class IdentifyBodyExtremity(tk.Frame):

  def __init__(self, parent, controller):
  
    tk.Frame.__init__(self, parent)
    self.controller = controller
    label = tk.Label(self, text="Prepare Config File", font=controller.title_font)
    label.pack(side="top", fill="x", pady=10)
    
    tk.Button(self, text="Click on the tip of the tail of the same zebrafish.", command=lambda: controller.chooseBodyExtremity(controller)).pack()
    
    tk.Label(self, text="Example:", font=("Helvetica", 10)).pack(side="top", fill="x", pady=10)
    
    canvas = Canvas(self, width=2500, height=2000)
    canvas.pack()
    img = PhotoImage(file="./code/GUI/blobExtremity.png")
    self.img = img
    canvas.create_image(20, 20, anchor=NW, image=img)
    
    tk.Button(self, text="Go to the start page", command=lambda: controller.show_frame("StartPage")).pack()


class GoToAdvanceSettings(tk.Frame):

  def __init__(self, parent, controller):
  
    tk.Frame.__init__(self, parent)
    self.controller = controller
    label = tk.Label(self, text="Prepare Config File", font=controller.title_font)
    label.pack(side="top", fill="x", pady=10)
    
    tk.Label(self, text="Do you want to detect bouts movements and/or further adjust tracking parameters?", font=("Helvetica", 12)).pack(side="top", fill="x", pady=10)
    
    yes = IntVar()
    Checkbutton(self, text="Yes", variable=yes).pack()
    tk.Label(self, text="").pack()
    
    no = IntVar()
    Checkbutton(self, text="No", variable=no).pack()
    tk.Label(self, text="").pack()
    
    tk.Button(self, text="Next", command=lambda: controller.goToAdvanceSettings(controller, yes.get(), no.get())).pack()
    
    tk.Button(self, text="Go to the start page", command=lambda: controller.show_frame("StartPage")).pack()


class FinishConfig(tk.Frame):

  def __init__(self, parent, controller):
  
    tk.Frame.__init__(self, parent)
    self.controller = controller
    
    tk.Label(self, text="Choose a name for the config file you want to create. Don't put any file extension here. For example, you could type: myconfig", font=("Helvetica", 10)).pack(side="top", fill="x", pady=10)
    
    configFileNameToSave = tk.Entry(self)
    configFileNameToSave.pack()
    
    if (globalVariables["mac"] or globalVariables["lin"]):
      tk.Button(self, text="Save Config File (this will also close this window, restart it afterwards)", command=lambda: controller.finishConfig(controller, configFileNameToSave.get())).pack()
    else:
      tk.Button(self, text="Save Config File", command=lambda: controller.finishConfig(controller, configFileNameToSave.get())).pack()




import sys
import os
import json
# import mat4py
import scipy.io as io
from scipy.interpolate import UnivariateSpline
import scipy.interpolate as interp
import numpy as np
from scipy.signal import find_peaks
import pandas as pd
import math
import popUpAlgoFollow
# from scipy.interpolate import splprep, splev, splrep

def IsMinOrMax(maxpeaks, minpeaks, val ):
  is_minOrMax = 0
  r = len(maxpeaks)
  for i in range(0,r):
    if maxpeaks[i] == val:
      is_minOrMax=1
  r = len(minpeaks)
  for i in range(0,r):
    if minpeaks[i] == val:
      is_minOrMax=1
  return is_minOrMax

def createSuperStruct(dataPerWell, wellPositions, hyperparameters):

  nbWells                      = hyperparameters["nbWells"]
  wellsAreRectangles           = hyperparameters["wellsAreRectangles"]
  tailAngleSmoothingFactor     = hyperparameters["tailAngleSmoothingFactor"]
  windowForLocalBendMinMaxFind = hyperparameters["windowForLocalBendMinMaxFind"]

  videoDataResults  = {}
  wellPoissMouv     = []
  
  for numWell in range(0,nbWells):
    
    item = {}
    
    j = dataPerWell[numWell]
    
    nbMouv = len(j)
    
    tab  = [[] for idAnimal in range(0, hyperparameters["nbAnimalsPerWell"])]
    # tab2 = []
    
    for i in range(0,nbMouv):
      item = j[i]
      
      angle_raw = item["TailAngle_Raw"]
      
      if len(angle_raw) > 10:
      
        rolling_window = hyperparameters["tailAngleMedianFilter"]
        if rolling_window > 0:
          shift = int(-rolling_window / 2)
          angle_median = np.array(pd.Series(angle_raw).rolling(rolling_window).median())
          angle_median = np.roll(angle_median, shift)
          for ii in range(0, rolling_window):
            angle_median[ii] = angle_raw[ii]
            # jj = rolling_window - 1 - ii
            # if math.isnan(angle_median[jj]):
              # angle_median[jj] = angle_median[jj+1]
          for ii in range(len(angle_median)-rolling_window,len(angle_median)):
            angle_median[ii] = angle_raw[ii]
            # if math.isnan(angle_median[ii]):
              # angle_median[ii] = angle_median[ii-1]
        else:
          angle_median = angle_raw
          
        # rolling_window = 3
        # if rolling_window > 0:
          # shift = int(-rolling_window / 2)
          # angle_median2 = np.array(pd.Series(angle_raw).rolling(rolling_window).median())
          # angle_median2 = np.roll(angle_median2, shift)
          # for ii in range(0, rolling_window):
            # jj = rolling_window - 1 - ii
            # if math.isnan(angle_median2[jj]):
              # angle_median2[jj] = angle_median2[jj+1]
          # for ii in range(len(angle_median2)-rolling_window,len(angle_median2)):
            # if math.isnan(angle_median2[ii]):
              # angle_median2[ii] = angle_median2[ii-1]
        # else:
          # angle_median2 = angle_raw
      
        x = np.linspace(0, 1, len(angle_median))
        
        # if True: # Original method
        s = UnivariateSpline(x, angle_median, s=tailAngleSmoothingFactor)
        TailAngle_smoothed = s(x) 
        # else:
          # tck = splrep(x, angle_median, s=0.01)
          # TailAngle_smoothed = splev(x, tck)
        
        # angle_median2 = np.zeros((len(angle_median),1))
        # angle_median2[:,0] = angle_median
        # print(angle_median2)

        # angle_median2 = np.zeros((1, len(angle_median)))
        # angle_median2[0,:] = angle_median
        # print(angle_median2)
        
        # angle_median2 = np.array(angle_median)
        # print(angle_median2)
        
        # tck, u = interp.splprep(angle_median2, s=0)
        # u = np.linspace(0, 1, len(angle_median2))
        # TailAngle_smoothed = np.column_stack(interp.splev(u, tck))
        
        item['TailAngle_smoothed'] = TailAngle_smoothed.tolist()
      else:
        TailAngle_smoothed = np.array(angle_raw)
        item['TailAngle_smoothed'] = angle_raw
      
      # maxDiffPeakToPeak = 0
      # for k in range(0, len(TailAngle_smoothed)):
        # wind = hyperparameters["windowForLocalBendMinMaxFind"]
        # if (k - wind >= 0) and (k + wind < len(TailAngle_smoothed)):
          # maxAngle = max([TailAngle_smoothed[k + l] for l in range(-wind, wind+1)])
          # minAngle = min([TailAngle_smoothed[k + l] for l in range(-wind, wind+1)])
          # if (maxAngle - minAngle > maxDiffPeakToPeak):
            # maxDiffPeakToPeak = maxAngle - minAngle
      
      if hyperparameters['extractAdvanceZebraParameters']:
      
        maxDiffPeakToPeak = 0
        maxAngle = max([TailAngle_smoothed[l] for l in range(0, len(TailAngle_smoothed))])
        minAngle = min([TailAngle_smoothed[l] for l in range(0, len(TailAngle_smoothed))])
        maxDiffPeakToPeak = maxAngle - minAngle
        
        minProminenceForBendsDetect = hyperparameters["minProminenceForBendsDetect"]
        if minProminenceForBendsDetect == -1:
          minProminenceForBendsDetect = maxDiffPeakToPeak / 10
          
        # TailAngle_smoothed = angle_median2
        maxpeaks, properties = find_peaks(TailAngle_smoothed, prominence=minProminenceForBendsDetect, width=hyperparameters["windowForLocalBendMinMaxFind"])
        minpeaks, properties = find_peaks(-TailAngle_smoothed, prominence=minProminenceForBendsDetect, width=hyperparameters["windowForLocalBendMinMaxFind"])
        
        if (len(minpeaks) + len(maxpeaks) >= hyperparameters["minNbPeaksForBoutDetect"] ):
        
          ind = 1
          minDiffBetweenSubsequentBendAmp = hyperparameters["minDiffBetweenSubsequentBendAmp"]
          
          Bend_Timing = []
          lastTailValue = 100000
          # Bend_TimingAbsolute(1)=floor((debut+fin)/2)

          for i in range(item['BoutStart'],item['BoutEnd']+1):
          
            minOrMax = IsMinOrMax(maxpeaks, minpeaks, i - item['BoutStart'] )
            
            if minOrMax:
              if (abs(lastTailValue - TailAngle_smoothed[i - item['BoutStart'] ]) > minDiffBetweenSubsequentBendAmp):
                Bend_Timing.append(i - item['BoutStart'] + 1) 
                lastTailValue = TailAngle_smoothed[i- item['BoutStart'] ]
            
            if (len(Bend_Timing) == 1) and (abs(lastTailValue) < hyperparameters["minFirstBendValue"]):
              Bend_Timing = []
              
          if len(Bend_Timing) > 3:
            Bend_Timing2 = []
            if hyperparameters["doubleCheckBendMinMaxStatus"]:
              # Checking first bend
              bendId = 0
              ind2 = Bend_Timing[bendId]
              ind3 = Bend_Timing[bendId+1]
              cond1 = (TailAngle_smoothed[0] < TailAngle_smoothed[ind2-1]) and (TailAngle_smoothed[ind3-1] < TailAngle_smoothed[ind2-1])
              cond2 = (TailAngle_smoothed[0] > TailAngle_smoothed[ind2-1]) and (TailAngle_smoothed[ind3-1] > TailAngle_smoothed[ind2-1])
              if cond1 or cond2:
                Bend_Timing2.append(ind2)
              # Checking bends in the middle
              for bendId in range(1, len(Bend_Timing)-1):
                ind1 = Bend_Timing[bendId-1]
                ind2 = Bend_Timing[bendId]
                ind3 = Bend_Timing[bendId+1]
                cond1 = (TailAngle_smoothed[ind1-1] < TailAngle_smoothed[ind2-1]) and (TailAngle_smoothed[ind3-1] < TailAngle_smoothed[ind2-1])
                cond2 = (TailAngle_smoothed[ind1-1] > TailAngle_smoothed[ind2-1]) and (TailAngle_smoothed[ind3-1] > TailAngle_smoothed[ind2-1])
                if cond1 or cond2:
                  Bend_Timing2.append(ind2)
              # Checking last bend
              bendId = len(Bend_Timing)-1
              ind1 = Bend_Timing[bendId-1]
              ind2 = Bend_Timing[bendId]
              n = len(TailAngle_smoothed)-1
              cond1 = (TailAngle_smoothed[ind1-1] < TailAngle_smoothed[ind2-1]) and (TailAngle_smoothed[n] < TailAngle_smoothed[ind2-1])
              cond2 = (TailAngle_smoothed[ind1-1] > TailAngle_smoothed[ind2-1]) and (TailAngle_smoothed[n] > TailAngle_smoothed[ind2-1])
              if cond1 or cond2:
                Bend_Timing2.append(ind2)
              Bend_Timing = Bend_Timing2
          
          if hyperparameters["removeFirstSmallBend"] and len(Bend_Timing) > 1:
            ind1 = Bend_Timing[0]
            ind2 = Bend_Timing[1]
            if abs(TailAngle_smoothed[ind1-1]) < abs(TailAngle_smoothed[ind2-1]) / hyperparameters["removeFirstSmallBend"]:
              Bend_Timing.pop(0)
          
          item['Bend_Timing'] = Bend_Timing
          
          Bend_TimingAbsolute = [val + item['BoutStart'] for val in Bend_Timing]
          
          item['Bend_TimingAbsolute'] = Bend_TimingAbsolute
          
          Bend_Amplitude = [TailAngle_smoothed[val-1] for val in Bend_Timing]
          item['Bend_Amplitude'] = Bend_Amplitude
            
          #####
          
          tab[item["AnimalNumber"]].append(item)
      else:
        item['TailAngle_raw'] = []
        item['TailAngle_smoothed'] = []
        tab[item["AnimalNumber"]].append(item)
    
    # tab2.append(tab)
    wellPoissMouv.append(tab)
  
  videoDataResults['wellPoissMouv'] = wellPoissMouv
  videoDataResults['wellPositions'] = wellPositions
  videoDataResults['firstFrame']    = hyperparameters["firstFrame"]
  
  path = hyperparameters["outputFolder"] + hyperparameters["videoName"] + '/results_' + hyperparameters["videoName"]
  
  # io.savemat(path + '.mat', videoDataResults)
  
  with open(path + '.txt', 'w') as outfile:
    json.dump(videoDataResults, outfile)
    
  if (hyperparameters["freqAlgoPosFollow"] != 0):
    print("Super Structure created")
  if hyperparameters["popUpAlgoFollow"]:
    popUpAlgoFollow.prepend("Super Structure created")

  return videoDataResults
  

# saving results in matlab file
# io.savemat('results.mat', videoDataResults2)





import h5py
import numpy as np
import cv2
import math
import json
import sys
import matplotlib.pyplot as plt
from scipy import interpolate
from getForegroundImage import getForegroundImage
from headEmbededFrame import headEmbededFrame
from scipy.interpolate import UnivariateSpline
from numpy import linspace

from getMidline import getMidline
from findTailExtremete import findTailExtremete
from findTheTwoSides import findTheTwoSides
from findBodyContour import findBodyContour
from Rotate import Rotate
from checkIfMidlineIsInBlob import checkIfMidlineIsInBlob

# from kalmanFilter import kalman_xy
# from trackingFunctions import calculateAngle
# from trackingFunctions import distBetweenThetas
# from trackingFunctions import assignValueIfBetweenRange

 
def tailTrackingExtremityDetect(headPosition,nbTailPoints,i,thresh1,frame,debugAdv,heading, hyperparameters):
  
  dst = frame.copy()
  dst = cv2.cvtColor(dst, cv2.COLOR_GRAY2RGB)
  firstFrame = hyperparameters["firstFrame"]
  lastFrame = hyperparameters["lastFrame"]
  
  if hyperparameters["debugTrackingThreshImg"]:
    cv2.imshow('debugTrackingThreshImg', thresh1)
    cv2.waitKey(0)
  
  # Finding blob corresponding to the body of the fish
  bodyContour = findBodyContour(headPosition, hyperparameters, thresh1)
  
  if type(bodyContour) != int:
    
    # Finding the two sides of the fish
    res = findTheTwoSides(headPosition, bodyContour, dst)
    
    # Finding tail extremity
    # if True:
    rotatedContour = bodyContour.copy()
    rotatedContour = Rotate(rotatedContour,int(headPosition[0]),int(headPosition[1]),heading,dst)
    [MostCurvyIndex, distance2] = findTailExtremete(rotatedContour, bodyContour, headPosition[0], int(res[0]), int(res[1]), debugAdv, dst, hyperparameters["tailExtremityMaxJugeDecreaseCoeff"])
    # else:
      # [MostCurvyIndex, distance2] = findTailExtremete(bodyContour, headPosition[0], int(res[0]), int(res[1]), debugAdv, dst)
    
    if debugAdv:
      # Head Center
      cv2.circle(dst, (int(headPosition[0]),int(headPosition[1])), 3, (255, 255, 0), -1)
      # Tail basis 1
      pt1 = bodyContour[int(res[0])][0]
      cv2.circle(dst, (pt1[0],pt1[1]), 3, (255, 0, 0), -1)
      # Tail basis 2
      pt1 = bodyContour[int(res[1])][0]
      cv2.circle(dst, (pt1[0],pt1[1]), 3, (255, 0, 0), -1)
      # Tail extremity
      pt1 = bodyContour[int(MostCurvyIndex)][0]
      cv2.circle(dst, (pt1[0],pt1[1]), 3, (0, 0, 255), -1)
      # Plotting points
      cv2.imshow('Frame', dst)
      cv2.waitKey(0)
    
    # Getting Midline
    taille = 10
    tail = getMidline(int(res[0]), int(res[1]), int(MostCurvyIndex), bodyContour, dst, taille-1, distance2, debugAdv, hyperparameters)
    
    # Optimizing midline if necessary
    midlineIsInBlobTrackingOptimization = hyperparameters["midlineIsInBlobTrackingOptimization"]
    if midlineIsInBlobTrackingOptimization:
      [allInside, tailLength] = checkIfMidlineIsInBlob(int(res[0]), int(res[1]), int(MostCurvyIndex), bodyContour, dst, taille-1, distance2, debugAdv, hyperparameters)
      if allInside == False:
        n = len(bodyContour)
        maxTailLength = -1
        for j in range(0, n):
          [allInside, tailLength] = checkIfMidlineIsInBlob(int(res[0]), int(res[1]), j, bodyContour, dst, taille-1, distance2, debugAdv, hyperparameters)
          if allInside:
            if tailLength > maxTailLength:
              MostCurvyIndex = j
              maxTailLength = tailLength
        tail = getMidline(int(res[0]), int(res[1]), int(MostCurvyIndex), bodyContour, dst, taille-1, distance2, debugAdv, hyperparameters)        
    
    # Applying snake on tail
    applySnake = False
    if applySnake:
      tail2 = tail[0]
      n = len(tail2)
      # tail2[n-1][0] = tail2[n-1][0] + (tail2[n-1][0] - tail2[n-2][0]) * 6
      # tail2[n-1][1] = tail2[n-1][1] + (tail2[n-1][1] - tail2[n-2][1]) * 6
      # print(type(tail))
      # r = np.linspace(tail2[0][0], tail2[0][0] + (tail2[1][0]-tail2[0][0]) * 15, 9)
      # c = np.linspace(tail2[0][1], tail2[0][1] + (tail2[1][1]-tail2[0][1]) * 15, 9)
      # tail2 = np.array([r, c]).T
      # r = np.linspace(tail2[0][0], tail2[n-1][0], 9)
      # c = np.linspace(tail2[0][1], tail2[n-1][1], 9)
      # tail2 = np.array([r, c]).T
      from skimage.color import rgb2gray
      from skimage.filters import gaussian
      from skimage.segmentation import active_contour
      snake = active_contour(gaussian(frame, 3), tail2, w_edge=-1000, bc="fixed")
      # snake = active_contour(gaussian(frame, 3), tail2, w_edge=0, bc="fixed-free")
      print(snake)
      # snake = tail2
      tail[0] = snake
  
  else:
    
    tail = np.zeros((1, 0, 2))
  
  # Inserting head position, smoothing tail and creating output
  tail = np.insert(tail, 0, headPosition, axis=1)
  # tail = smoothTail(tail, nbTailPoints)
  
  output = np.zeros((1, len(tail[0]), 2))
  
  for idx, x in enumerate(tail[0]):
    output[0][idx][0] = x[0]
    output[0][idx][1] = x[1]

  return output



import tkinter as tk
from tkinter import font  as tkfont
from tkinter import filedialog
from tkinter import ttk
from tkinter import *


class HeadEmbeded(tk.Frame):

  def __init__(self, parent, controller):
  
    tk.Frame.__init__(self, parent)
    self.controller = controller
    label = tk.Label(self, text="Prepare Config File", font=controller.title_font)
    label.pack(side="top", fill="x", pady=10)
    
    #####
    
    tk.Label(self, text="Choose only one of the options below:", font=("Helvetica", 12)).pack(side="top", fill="x", pady=10)
    
    blackBack = IntVar()
    Checkbutton(self, text="Black background, white zebrafish.", variable=blackBack).pack()
    
    whiteBack = IntVar()
    Checkbutton(self, text="White background, black zebrafish.", variable=whiteBack).pack()
    
    tk.Label(self, text="").pack()
    
    #####
    
    tk.Label(self, text="Do you want ZebraZoom to detect bouts of movement?", font=("Helvetica", 12)).pack(side="top", fill="x", pady=10)
    
    noBoutDetect = IntVar()
    Checkbutton(self, text="No. I want the tracking data for all frames of the videos.", variable=noBoutDetect).pack()
    
    boutDetection = IntVar()
    Checkbutton(self, text="Yes. I want the tracking data only when the fish is moving.", variable=boutDetection).pack()
    
    tk.Label(self, text="").pack()
    
    #####

    tk.Label(self, text="Do you want to try to tweak tracking parameters further?", font=("Helvetica", 12)).pack(side="top", fill="x", pady=10)
    
    tk.Label(self, text="Warning: further tweaking tracking parameters could make tracking results worse; please try without this option first.").pack()
    
    tweakTrackingParamsYes = IntVar()
    Checkbutton(self, text="Yes", variable=tweakTrackingParamsYes).pack()
    
    tweakTrackingParamsNo  = IntVar()
    Checkbutton(self, text="No", variable=tweakTrackingParamsNo).pack()
    tk.Label(self, text="").pack()
    
    # tk.Label(self, text="Options: Only select one or several of the options below if you've already tried the tracking without them:", font=("Helvetica", 12)).pack(side="top", fill="x", pady=10)
    
    # optionExtendedDescentSearchOption = IntVar()
    # Checkbutton(self, text="Extended descent search option", variable=optionExtendedDescentSearchOption).pack()
    # tk.Label(self, text="Warning: in some cases, this will make the tracking work better; please try without this option first.").pack()
    # tk.Label(self, text="").pack()
    
    # optionBackgroundExtractionOption  = IntVar()
    # Checkbutton(self, text="Background extraction option", variable=optionBackgroundExtractionOption).pack()
    # tk.Label(self, text="Warning: in some cases, this will make the tracking work better; please try without this option first.").pack()
    # tk.Label(self, text="").pack()
    
    #####
    
    tk.Button(self, text="Next", command=lambda: controller.headEmbededGUI(controller, blackBack.get(), whiteBack.get(), noBoutDetect.get(), boutDetection.get(), tweakTrackingParamsYes.get(), tweakTrackingParamsNo.get())).pack()
    
    tk.Button(self, text="Go to the start page", command=lambda: controller.show_frame("StartPage")).pack()




import h5py
import numpy as np
import cv2
import math
import json
import os
import sys
import popUpAlgoFollow

import numpy as np
from filterpy.kalman import KalmanFilter
from filterpy.common import Q_discrete_white_noise

# from calculateCurvature import calculateCurvature
from detectMovementWithRawVideo import detectMovementWithRawVideo
from getTailTipManual import getHeadPositionByFileSaved, getTailTipByFileSaved

import pdb

def distBetweenThetas(theta1, theta2):
  diff = 0
  if theta1 > theta2:
    diff = theta1 - theta2
  else:
    diff = theta2 - theta1
  if diff > math.pi:
    diff = (2 * math.pi) - diff
  return diff

def calculateAngle(vectStart, vectEnd):
  x = vectEnd[0] - vectStart[0]
  y = vectEnd[1] - vectStart[1]
  if x == 0:
    if y > 0:
      heading = math.pi / 2
    else:
      heading = (3 * math.pi) / 2
  else:
    heading = np.arctan(abs(y/x))
    if (x < 0) and (y > 0):
      heading = math.pi - heading
    elif (x < 0) and (y < 0):
      heading = heading + math.pi
    elif (x < 0) and (y == 0):
      heading = math.pi
    elif (x > 0) and (y < 0):
      heading = 2*math.pi - heading
  return heading
  
def calculateTailAngle(angle1, angle2):
  output = angle1 - angle2
  output = (output + 2 * 3.14159265) % (2 * 3.14159265)
  if output > 3.14159265:
    output = output - 2*3.14159265;
  return output

def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, wellPositions, background):

  firstFrame = hyperparameters["firstFrame"]
  thresAngleBoutDetect = hyperparameters["thresAngleBoutDetect"]
  debugExtractParams = hyperparameters["debugExtractParams"]

  trackingTailAllAnimals = trackingData[0]
  trackingHeadingAllAnimals = trackingData[1]
  headPosition = trackingData[2]
  tailTip = trackingData[3]
  
  data = []
  
  for animalId in range(0, len(trackingTailAllAnimals)):
    
    trackingTail    = trackingTailAllAnimals[animalId]
    trackingHeading = trackingHeadingAllAnimals[animalId]
    
    n = len(trackingTail[0])
    
    debug = 0

    nbFrames = len(trackingTail)
    nbPoints = len(trackingTail[0])

    tail_1  = np.zeros((nbFrames, 2))
    tip     = np.zeros((nbFrames, 2))
    head    = np.zeros((nbFrames, 2))
    heading = np.zeros((nbFrames, 1))
    angle   = np.zeros((nbFrames, 1))
    tailX   = np.zeros((nbFrames, n))
    tailY   = np.zeros((nbFrames, n))
    
    # fTipX = KalmanFilter (dim_x=2, dim_z=1)
    # fTipX.x = np.array([[2.],[0.]])
    # fTipX.F = np.array([[1.,1.],[0.,1.]])
    # fTipX.H = np.array([[1.,0.]])
    # fTipX.P = np.array([[1000.,    0.],[   0., 1000.] ])
    # fTipX.R = np.array([[5.]])
    # fTipX.Q = Q_discrete_white_noise(dim=2, dt=0.1, var=0.13)

    # fTipY = KalmanFilter (dim_x=2, dim_z=1)
    # fTipY.x = np.array([[2.],[0.]])
    # fTipY.F = np.array([[1.,1.],[0.,1.]])
    # fTipY.H = np.array([[1.,0.]])
    # fTipY.P = np.array([[1000.,    0.],[   0., 1000.] ])
    # fTipY.R = np.array([[5.]])
    # fTipY.Q = Q_discrete_white_noise(dim=2, dt=0.1, var=0.13)
    
    if hyperparameters["headingCalculationMethod"] == "calculatedWithMedianTailTip":
      tip2 = np.zeros((nbFrames, 2))
      nbTailPoints = len(trackingTail[0]) - 1
      for i in range(0,nbFrames):
        tip2[i] = np.array([ trackingTail[i][nbTailPoints][0], trackingTail[i][nbTailPoints][1] ])
      medianTip = np.median(tip2,axis=0)
      
    curvatureBef = 0
    
    for i in range(0,nbFrames):
    
      if (hyperparameters["freqAlgoPosFollow"] != 0) and (i % hyperparameters["freqAlgoPosFollow"] == 0):
        print("Extract Param Begin: wellNumber:",wellNumber," ; frame:",i)
      
      tail_1[i]  = np.array([ trackingTail[i][1][0], trackingTail[i][1][1] ])
      
      nbTailPoints = len(trackingTail[0]) - 1
      tip[i]     = np.array([ trackingTail[i][nbTailPoints][0], trackingTail[i][nbTailPoints][1] ])
      
      # print("bef:",tip[i])
      # fTipX.predict()
      # fTipX.update(np.array([tip[i][0]]))
      # tip[i][0] = fTipX.x[0]
      # trackingTail[i][nbTailPoints][0] = fTipX.x[0]
      
      # fTipY.predict()
      # fTipY.update(np.array([tip[i][1]]))
      # tip[i][1] = fTipY.x[0]
      # trackingTail[i][nbTailPoints][1] = fTipY.x[0]
      # print("aft:",tip[i])
      
      head[i]    = np.array([ trackingTail[i][0][0], trackingTail[i][0][1] ])
      
      if hyperparameters["headingCalculationMethod"] == "calculatedWithFirstTailPt":
        heading[i] = calculateAngle(head[i], tail_1[i])
      elif hyperparameters["headingCalculationMethod"] == "calculatedWithMedianTailTip":
        heading[i] = calculateAngle(head[i], medianTip)
      else: # calculatedWithHead
        heading[i] = trackingHeading[i]
        diff1 = distBetweenThetas(heading[i],           calculateAngle(head[i], tail_1[i]))
        diff2 = distBetweenThetas(heading[i] + math.pi, calculateAngle(head[i], tail_1[i]))
        if diff2 < diff1:
          heading[i] = heading[i] + math.pi
      
      angle[i] = calculateTailAngle(calculateAngle(head[i], tip[i]), heading[i])
      
      heading[i] = (heading[i] + math.pi) % (2*math.pi)
      
      tailX[i]   = trackingTail[i,:,0]
      tailY[i]   = trackingTail[i,:,1]

      # if False:
        # tailForCurv = np.zeros((10, 2))
        # for j in range(0,10):
          # tailForCurv[j][0] = tailX[i][j]
          # tailForCurv[j][1] = tailY[i][j]
        # curvature = calculateCurvature(tailForCurv)
        # diff = np.max(np.abs(curvature))
        # curvatureBef = curvature
    
    if hyperparameters["noBoutsDetection"] == 1:
      auDessus        = np.zeros((nbFrames, 1))
      for i in range(0,nbFrames):
        auDessus[i] = 1
    else:
      if hyperparameters["boutEdgesWhereZeros"] == 1:
        auDessus        = np.zeros((nbFrames, 1))
        for i in range(0,nbFrames):
          if ((head[i][0] == 0) and (head[i][1] == 0)):
            auDessus[i] = 0
          else:
            auDessus[i] = 1
      else:
        if hyperparameters["thresForDetectMovementWithRawVideo"] == 0:
          window = 5
          angleVariation  = np.zeros((nbFrames, 1))
          auDessus        = np.zeros((nbFrames, 1))
          for i in range(0,nbFrames):
            if (hyperparameters["freqAlgoPosFollow"] != 0) and (i % (hyperparameters["freqAlgoPosFollow"]) == 0):
              print("Extract Param Middle: wellNumber:",wellNumber," ; frame:",i)
            min = 10000
            max = -10000
            for j in range(i-2*window,i+1):
              if (j >= 0) and (j<nbFrames):
                if angle[j] > max:
                  max = angle[j]
                if angle[j] < min:
                  min = angle[j]
            angleVariation[i] = max - min
            if debug:
              print(i,angleVariation[i])
            if angleVariation[i] > thresAngleBoutDetect:
              auDessus[i] = 1
        else:
          auDessus = detectMovementWithRawVideo(hyperparameters, videoPath, background, wellNumber, wellPositions, head, headPosition, tailTip)
    
    if (hyperparameters["freqAlgoPosFollow"] != 0):
      print("Extract Param Middle: wellNumber:",wellNumber," ; frame:",i)
    
    auDessus2 = np.copy(auDessus)
    if hyperparameters["boutEdgesWhereZeros"] == 0 and hyperparameters["noBoutsDetection"] == 0:
      windowGap = hyperparameters["fillGapFrameNb"]
      if windowGap:
        for i in range(windowGap,nbFrames-windowGap):
          if auDessus[i] == 0:
            j = i - windowGap
            while (auDessus[j] == 0) and (j < i + windowGap):
              j = j + 1
            if j < i + windowGap:
              j = j + 1;
              while (auDessus[j] == 0) and (j < i + windowGap):
                j = j + 1;
              if auDessus[j] > 0:
                for k in range(i - windowGap, i + windowGap):
                  auDessus2[k] = 1
    
    bouts   = np.zeros((0, 3))
    curBout = np.zeros((1, 3))
    position = 0
    
    for i in range(0, nbFrames):
    
      if (hyperparameters["freqAlgoPosFollow"] != 0) and (i % (hyperparameters["freqAlgoPosFollow"]*10) == 0):
        print("Extract Param End1 Freq*10 : wellNumber:",wellNumber," ; frame:",i)
    
      if (position==0) and (auDessus2[i]==1):
        curBout[0][0] = wellNumber
        curBout[0][1] = i
        position = 1
        retire = 1
      if (position == 1) and ((auDessus2[i] == 0) or (i == nbFrames-1)):
        curBout[0][2] = i-1
        position = 0
        if curBout[0][2] - curBout[0][1] >= hyperparameters["detectBoutMinNbFrames"] or hyperparameters["noChecksForBoutSelectionInExtractParams"]:
          debMouv = int(curBout[0][1])
          endMouv = int(curBout[0][2])
          dist = math.sqrt( (head[debMouv,0]-head[endMouv,0])**2 + (head[debMouv,1]-head[endMouv,1])**2 )
          if ((dist >= hyperparameters["detectBoutMinDist"]) and (np.max(angle[debMouv:endMouv])-np.min(angle[debMouv:endMouv]) >= hyperparameters["detectBoutMinAngleDiff"])) or hyperparameters["noChecksForBoutSelectionInExtractParams"]:
            bouts = np.append(bouts, curBout, axis=0)
          else:
            if debugExtractParams:
              print("Bout starting at", debMouv, " has a dist or angle diff too big. Dist:", dist," minimum was:", hyperparameters["detectBoutMinDist"],". DetectBoutMinAngleDiff:",np.max(angle[debMouv:endMouv])-np.min(angle[debMouv:endMouv]),", minimmum was:",hyperparameters["detectBoutMinAngleDiff"])
        else:
          if debugExtractParams:
            print("Bout starting at", int(curBout[0][1]), "was too small. Length:", curBout[0][2] - curBout[0][1],". Minimum was:", hyperparameters["detectBoutMinNbFrames"])
    
    # Refining beginning and end of bout detection to remove the artefacts from the fill gap procedure
    for numBout in range(0,len(bouts)):
      debMouv = int(bouts[numBout][1])
      finMouv = int(bouts[numBout][2])
      # Refining the beginning
      pos = debMouv
      while int(auDessus[pos]) == 0:
        pos = pos + 1
      bouts[numBout][1] = pos
      # Refining the end
      pos = finMouv
      while int(auDessus[pos]) == 0:
        pos = pos - 1
      bouts[numBout][2] = pos
    
    for i in range(0,len(bouts)):
    
      if (hyperparameters["freqAlgoPosFollow"] != 0) and (i % (hyperparameters["freqAlgoPosFollow"]*10) == 0):
        print("Extract Param End2 Freq*10 : wellNumber:",wellNumber," ; frame:",i)
      
      item = {}
      start = int(bouts[i][1])
      end   = int(bouts[i][2])
      item["AnimalNumber"]  = animalId #int(bouts[i][0])
      item["BoutStart"]     = start + firstFrame
      item["BoutEnd"]       = end + firstFrame
      item["TailAngle_Raw"] = angle[start:end+1,0].tolist()
      item["HeadX"]         = head[start:end+1,0].tolist()
      item["HeadY"]         = head[start:end+1,1].tolist()
      # item["Heading_raw"]   = heading[start:end,0]
      item["Heading"]       = heading[start:end+1,0].tolist()
      item["TailX_VideoReferential"] = tailX[start:end+1].tolist()
      item["TailY_VideoReferential"] = tailY[start:end+1].tolist()
      # item["TailX_HeadingReferential"]
      # item["TailY_HeadingReferential"]
      data.append(item)
  
  print("Parameters extracted for well",wellNumber)
  if hyperparameters["popUpAlgoFollow"]:
    popUpAlgoFollow.prepend("Parameters extracted for well " + str(wellNumber))
  
  return data



import cv2
import os
import shutil
import math
import matplotlib.pyplot as plt
import numpy as np
import time
import pandas as pd

def perBoutOutput(superStruct, hyperparameters, videoName):
  
  # Creation of the sub-folder "perBoutOutput" 
  if os.path.exists(hyperparameters["outputFolder"] + videoName + "/perBoutOutput"):
    shutil.rmtree(hyperparameters["outputFolder"] + videoName + "/perBoutOutput")
  while True:
    try:
      os.mkdir(hyperparameters["outputFolder"] + videoName + "/perBoutOutput")
      break
    except OSError as e:
      print("waiting inside except")
      time.sleep(0.1)
    else:
      print("waiting")
      time.sleep(0.1)
      
  # opening the video previously created in createValidationVideo as an input video stream
  cap = cv2.VideoCapture(hyperparameters["outputFolder"] + hyperparameters["videoName"] + '/' + hyperparameters["videoName"] + '.avi')
  if (cap.isOpened()== False): 
    print("Error opening video stream or file")
  
  # Video frame start and end
  firstFrame = hyperparameters["firstFrame"]
  lastFrame  = hyperparameters["lastFrame"]
  nbFrames   = lastFrame - firstFrame + 1
  perBoutOutputVideoStartStopFrameMargin = hyperparameters["perBoutOutputVideoStartStopFrameMargin"]
  
  # Getting well positions
  infoWells = []
  for i in range(0, len(superStruct["wellPositions"])):
    x = superStruct["wellPositions"][i]["topLeftX"]
    y = superStruct["wellPositions"][i]["topLeftY"]
    lengthX = superStruct["wellPositions"][i]["lengthX"]
    lengthY = superStruct["wellPositions"][i]["lengthY"]
    infoWells.append([x, y, lengthX, lengthY])
  
  # Going through each well, each fish and each bout
  for i in range(0, len(superStruct["wellPoissMouv"])):
    for j in range(0, len(superStruct["wellPoissMouv"][i])):
      for k in range(0, len(superStruct["wellPoissMouv"][i][j])):
        
        # Creation of the curvature graph for bout k
        TailX_VideoReferential = superStruct["wellPoissMouv"][i][j][k]["TailX_VideoReferential"]
        TailY_VideoReferential = superStruct["wellPoissMouv"][i][j][k]["TailY_VideoReferential"]
        
        curvature = []
        for l in range(0, len(TailX_VideoReferential)):
          tailX = TailX_VideoReferential[l]
          tailY = TailY_VideoReferential[l]
          # curvature calculation method 1
          # a = np.array([[tailX[m], tailY[m]] for m in range(5, len(tailX))]) # CHECK OUT THE 5 HERE !!!
          # dx_dt = np.gradient(a[:, 0])
          # dy_dt = np.gradient(a[:, 1])
          # velocity = np.array([ [dx_dt[i], dy_dt[i]] for i in range(dx_dt.size)])
          # ds_dt = np.sqrt(dx_dt * dx_dt + dy_dt * dy_dt)
          # tangent = np.array([1/ds_dt] * 2).transpose() * velocity
          # tangent_x = tangent[:, 0]
          # tangent_y = tangent[:, 1]
          # deriv_tangent_x = np.gradient(tangent_x)
          # deriv_tangent_y = np.gradient(tangent_y)
          # dT_dt = np.array([ [deriv_tangent_x[i], deriv_tangent_y[i]] for i in range(deriv_tangent_x.size)])
          # length_dT_dt = np.sqrt(deriv_tangent_x * deriv_tangent_x + deriv_tangent_y * deriv_tangent_y)
          # normal = np.array([1/length_dT_dt] * 2).transpose() * dT_dt
          # d2s_dt2 = np.gradient(ds_dt)
          # d2x_dt2 = np.gradient(dx_dt)
          # d2y_dt2 = np.gradient(dy_dt)
          # curv = np.abs(d2x_dt2 * dy_dt - dx_dt * d2y_dt2) / (dx_dt * dx_dt + dy_dt * dy_dt)**1.5
          # curvature calculation method 2
          ydiff  = np.diff(tailY)
          ydiff2 = np.diff(ydiff)
          xdiff  = np.diff(tailX)
          xdiff2 = np.diff(xdiff)
          curv = xdiff2
          l = len(curv)
          av = 0
          for ii in range(0, l-1):
            num = xdiff[ii] * ydiff2[ii] - ydiff[ii] * xdiff2[ii]
            den = (xdiff[ii]**2 + ydiff[ii]**2)**1.5
            curv[ii] = num / den
          curv = curv[1:l-2]  
          curvature.append(curv)
          
        rolling_window = hyperparameters["curvatureMedianFilterSmoothingWindow"]
        curvature2 = []
        curvatureTransposed = np.transpose(curvature)
        for transposedCurv in curvatureTransposed:
          # x = [i for i in range(0, len(transposedCurv))]
          # tck = splrep(x, transposedCurv, s=0.0001)
          # smoothedTransposedCurv = splev(x, tck)
          shift = int(-rolling_window / 2)
          smoothedTransposedCurv = np.array(pd.Series(transposedCurv).rolling(rolling_window).median())
          smoothedTransposedCurv = np.roll(smoothedTransposedCurv, shift)
          for ii in range(0, rolling_window):
            smoothedTransposedCurv[ii] = transposedCurv[ii]
          for ii in range(len(smoothedTransposedCurv)-rolling_window,len(smoothedTransposedCurv)):
            smoothedTransposedCurv[ii] = transposedCurv[ii] 
          curvature2.append(smoothedTransposedCurv)  
        curvature = np.transpose(curvature2)
          
        fig = plt.figure(1)
        plt.pcolor(curvature)
        
        ax = fig.axes
        ax[0].set_xlabel('Rostral to Caudal')
        ax[0].set_ylabel('Frame number')
        plt.colorbar()
        plt.savefig(hyperparameters["outputFolder"] + hyperparameters["videoName"] + '/perBoutOutput/' + hyperparameters["videoName"] + "_curvature_bout" + str(k) + '.png')
        plt.close(1)
        
        
        # Creation of tail angle graph for bout k
        tailAngle = superStruct["wellPoissMouv"][i][j][k]["TailAngle_smoothed"]
        plt.figure(1)
        bStart = superStruct["wellPoissMouv"][i][j][k]["BoutStart"]
        bEnd   = superStruct["wellPoissMouv"][i][j][k]["BoutEnd"]
        plt.plot([i for i in range(bStart, bEnd + 1)], [t*(180/math.pi) for t in tailAngle])
        if hyperparameters["perBoutOutputYaxis"]:
          plt.axis([bStart, bEnd + 1, hyperparameters["perBoutOutputYaxis"][0], hyperparameters["perBoutOutputYaxis"][1]])
        plt.savefig(hyperparameters["outputFolder"] + hyperparameters["videoName"] + '/perBoutOutput/' + hyperparameters["videoName"] + "_angle_bout" + str(k) + '.png')
        plt.close(1)
        
        # Creation of sub-video for bout k
        
        x0 = superStruct["wellPoissMouv"][i][j][k]["HeadX"][0]
        y0 = superStruct["wellPoissMouv"][i][j][k]["HeadY"][0]
        TailX_VideoReferential = superStruct["wellPoissMouv"][i][j][k]["TailX_VideoReferential"]
        TailY_VideoReferential = superStruct["wellPoissMouv"][i][j][k]["TailY_VideoReferential"]
        tailExtX = TailX_VideoReferential[0][len(TailX_VideoReferential[0])-1]
        tailExtY = TailY_VideoReferential[0][len(TailY_VideoReferential[0])-1]
        dist = int((math.sqrt((tailExtX-x0)**2 + (tailExtY-y0)**2))*1.3)
        
        outputName = hyperparameters["outputFolder"] + hyperparameters["videoName"] + '/perBoutOutput/' + hyperparameters["videoName"] + "_bout" + str(k) + '.avi'
        outputVideoX = int(x+dist/2)-int(x-dist/2)
        outputVideoY = int(dist)
        out = cv2.VideoWriter(outputName,cv2.VideoWriter_fourcc('M','J','P','G'), 10, (outputVideoX, outputVideoY))
        
        BoutStart = superStruct["wellPoissMouv"][i][j][k]["BoutStart"] - perBoutOutputVideoStartStopFrameMargin
        BoutEnd = superStruct["wellPoissMouv"][i][j][k]["BoutEnd"] + perBoutOutputVideoStartStopFrameMargin
        if BoutStart < 0:
          BoutStart = 0
        if BoutEnd >= nbFrames:
          BoutEnd = nbFrames - 1
        mouvLength = len(superStruct["wellPoissMouv"][i][j][k]["HeadX"])
        cap.set(1, BoutStart)
        
        for l in range(BoutStart, BoutEnd):
          ret, frame = cap.read()
          if l < superStruct["wellPoissMouv"][i][j][k]["BoutStart"]:
            l2 = 0
          elif l >= superStruct["wellPoissMouv"][i][j][k]["BoutEnd"]:
            l2 = mouvLength - 1
          else:
            l2 = l - superStruct["wellPoissMouv"][i][j][k]["BoutStart"]
          x = superStruct["wellPoissMouv"][i][j][k]["HeadX"][l2]
          y = superStruct["wellPoissMouv"][i][j][k]["HeadY"][l2]
          Heading = superStruct["wellPoissMouv"][i][j][k]["Heading"][l2]
          
          x = int(x + infoWells[i][0])
          y = int(y + infoWells[i][1])
          
          rows = len(frame)
          cols = len(frame[0])
          M = cv2.getRotationMatrix2D((x, y), -(((math.pi/2) - (Heading+math.pi))%(2*math.pi))*(180/math.pi), 1)
          frame = cv2.warpAffine(frame, M, (cols,rows))
          frame = frame[int(y):int(y)+outputVideoY, int(x-dist/2):int(x-dist/2)+outputVideoX]
          frame2 = np.zeros((outputVideoX, outputVideoY, 3), np.uint8)
          frame2[0:len(frame), 0:len(frame[0]), :] = frame[0:len(frame), 0:len(frame[0]), :]
          frame = frame2
          
          out.write(frame)
          
        out.release()



import numpy as np
import cv2
from trackingFunctions import calculateAngle
from trackingFunctions import distBetweenThetas
from trackingFunctions import assignValueIfBetweenRange
import math
from scipy.interpolate import UnivariateSpline
from numpy import linspace
import os.path
import csv
from scipy.interpolate import interp1d
from scipy.interpolate import splprep, splev


def smoothTail(points, nbTailPoints, smoothingFactor):
  
  points3 = [np.array(points[0]), np.array(points[1])]
  
  y = points[0]
  x = linspace(0, 1, len(y))
    
  if len(x) > 3:
   
    interpolated_points = {}
    
    try:
      tck, u = splprep(points3, s=smoothingFactor)
      u2 = [i/(nbTailPoints-1) for i in range(0, nbTailPoints)]
      interpolated_points = splev(u2, tck)
      newX = interpolated_points[0]
      newY = interpolated_points[1]
    except:
      newX = points[0]
      newY = points[1]
    
  else:
    newX = points[0]
    newY = points[1]
  
  return [newX, newY]


def interpolateTail(points, nbTailPoints):

  y = points[0]
  x = linspace(0, 1, len(y))
    
  if len(x) > 3:
    
    points2 = []
    for i in range(0, len(points[0])-1):
      if not((points[0][i] == points[0][i+1]) and (points[1][i] == points[1][i+1])):
        points2.append([points[0][i], points[1][i]])

    i = len(points[0]) - 1
    if not((points[0][i-1] == points[0][i]) and (points[1][i-1] == points[1][i])):
      points2.append([points[0][i], points[1][i]])
        
    points = np.array(points2).T
    
    # Define some points:
    points = np.array([points[0], points[1]]).T  # a (nbre_points x nbre_dim) array

    # Linear length along the line:
    distance = np.cumsum( np.sqrt(np.sum( np.diff(points, axis=0)**2, axis=1 )) )
    distance = np.insert(distance, 0, 0)/distance[-1]

    # Interpolation for different methods:
    interpolation_method = 'quadratic'    # 'slinear', 'quadratic', 'cubic'
    alpha = np.linspace(0, 1, nbTailPoints)

    interpolated_points = {}
    
    interpolator =  interp1d(distance, points, kind=interpolation_method, axis=0)
    interpolated_points = interpolator(alpha)
    
    interpolated_points = interpolated_points.T
    
    newX = interpolated_points[0]
    newY = interpolated_points[1]
    
  else:
    
    newX = points[0]
    newY = points[1]
  
  return [newX, newY]


def appendPoint(x, y, points):
  curPoint = np.zeros((2, 1))
  curPoint[0] = x;
  curPoint[1] = y;
  points = np.append(points, curPoint, axis=1)
  return points

def findNextPoints(depth,x,y,frame,points,angle,maxDepth,steps,nbList,initialImage,debug, hyperparameters):
  
  lenX = len(frame[0]) - 1
  lenY = len(frame) - 1

  thetaDiffAccept = 1
  
  if depth < hyperparameters["initialTailPortionMaxSegmentDiffAngleCutOffPos"] * maxDepth:
    thetaDiffAccept = hyperparameters["initialTailPortionMaxSegmentDiffAngleValue"]
  
  if depth > 0.85*maxDepth:
    thetaDiffAccept = 0.6
  
  pixTotMax = 1000000
  maxTheta  = angle

  l = [i*(math.pi/nbList) for i in range(0,2*nbList) if distBetweenThetas(i*(math.pi/nbList), angle) < thetaDiffAccept]
  
  # if debug:
    # print("debug")
  
  for step in steps:
  
    for theta in l:
      
      xNew = assignValueIfBetweenRange(int(x + step * (math.cos(theta))), 0, lenX)
      yNew = assignValueIfBetweenRange(int(y + step * (math.sin(theta))), 0, lenY)
      pixTot = frame[yNew][xNew]
      
      # if debug:
        # print([theta,pixTot])
      
      # Keeps that theta angle as maximum if appropriate
      if (pixTot < pixTotMax):
        pixTotMax = pixTot
        maxTheta = theta
        xTot = xNew
        yTot = yNew
  
  w = 4
  ym = yTot - w
  yM = yTot + w
  xm = xTot - w
  xM = xTot + w
  if ym < 0:
    ym = 0
  if xm < 0:
    xm = 0
  if yM > len(initialImage):
    yM = len(initialImage)
  if xM > len(initialImage[0]):
    xM = len(initialImage[0])
  
  pixSur = np.min(frame[ym:yM, xm:xM]) #initialImage[ym:yM, xm:xM])
  # if debug:
    # print("depth:", depth, " ; maxDepth:", maxDepth, " ; pixSur:", pixSur)
  
  # if depth > 0.95*maxDepth:
    # pixTot = frame[y][x]
    # if (pixTot < pixTotMax):
      # pixTotMax = pixTot
      # maxTheta = theta
      # xTot = x
      # yTot = y
      # depth = maxDepth + 10
  
  # if debug:
    # print(["max:",maxTheta,pixTotMax])
  
  # Calculates distance between new and old point
  distSubsquentPoints = math.sqrt((xTot - x)**2 + (yTot - y)**2)
  
  pixSurMax = hyperparameters["headEmbededParamTailDescentPixThreshStop"]
  # pixSurMax = 220 #150 #245 #150
  if ((pixSur < pixSurMax) or (depth < hyperparameters["authorizedRelativeLengthTailEnd"]*maxDepth)):
    points = appendPoint(xTot, yTot, points)
    if debug:
      cv2.circle(frame, (xTot, yTot), 3, (255,0,0),   -1)
      cv2.imshow('HeadEmbeddedTailTracking', frame)
      cv2.waitKey(0)
    
  newTheta = calculateAngle(x,y,xTot,yTot)
  if distSubsquentPoints > 0 and depth < maxDepth and ((pixSur < pixSurMax) or (depth < hyperparameters["authorizedRelativeLengthTailEnd"]*maxDepth)):
    (points,nop) = findNextPoints(depth+distSubsquentPoints,xTot,yTot,frame,points,newTheta,maxDepth,steps,nbList,initialImage,debug, hyperparameters)
  
  return (points,newTheta)


def headEmbededTailTracking(headPosition,nbTailPoints,i,thresh1,frame,hyperparameters,heading,maxDepth,tailTip):
  steps   = hyperparameters["step"]
  nbList  = 10
  
  x = headPosition[0]
  y = headPosition[1]
  
  initialImage = frame.copy()
  
  gaussian_blur = hyperparameters["headEmbededParamGaussianBlur"]
  frame = cv2.GaussianBlur(frame, (gaussian_blur, gaussian_blur), 0)
  # angle = hyperparameters["headEmbededParamInitialAngle"]
  angle = calculateAngle(x, y, tailTip[0], tailTip[1])
  
  points = np.zeros((2, 0))
  (points, lastFirstTheta2) = findNextPoints(0,x,y,frame,points,angle,maxDepth,steps,nbList,initialImage,hyperparameters["debugHeadEmbededFindNextPoints"], hyperparameters)
  points = np.insert(points, 0, headPosition, axis=1) 
  
  if len(points[0]) > 3:
    if hyperparameters["smoothTailHeadEmbeded"]:
      points = smoothTail(points, nbTailPoints, hyperparameters["smoothTailHeadEmbeded"])
    else:
      if not(hyperparameters["adjustHeadEmbededTracking"]):
        points   = interpolateTail(points, nbTailPoints)
      else:
        nDist    = len(points[0]) - 1
        stepDist = 10 / nDist
        tab      = [int((i/9)*nDist) for i in range(0,10)]
        points   = [[points[0][i] for i in tab], [points[1][i] for i in tab]]
  
  output = np.zeros((1, len(points[0]), 2))

  for idx, x in enumerate(points[0]):
    output[0][idx][0] = x
    output[0][idx][1] = points[1][idx]

  return output


def headEmbededTailTrackFindMaxDepth(headPosition,nbTailPoints,i,x,y,thresh1,frame,hyperparameters,oppHeading,tailTip):

  headEmbededParamTailDescentPixThreshStopInit = hyperparameters["headEmbededParamTailDescentPixThreshStop"]
  hyperparameters["headEmbededParamTailDescentPixThreshStop"] = 256
  
  x = headPosition[0]
  y = headPosition[1]

  steps   = hyperparameters["step"]
  nbList  = 10
  
  initialImage = frame.copy()
  
  gaussian_blur = hyperparameters["headEmbededParamGaussianBlur"]
  frame = cv2.GaussianBlur(frame, (gaussian_blur, gaussian_blur), 0)
  
  angle = calculateAngle(x, y, tailTip[0], tailTip[1])
  
  points = np.zeros((2, 0))
  
  (points, lastFirstTheta2) = findNextPoints(0,x,y,frame,points,angle,hyperparameters["headEmbededTailTrackFindMaxDepthInitialMaxDepth"],steps,nbList,initialImage, hyperparameters["debugHeadEmbededFindNextPoints"], hyperparameters)
  
  distToTip        = np.full((200),10000)
  curTailLengthTab = np.full((200),10000)
  curTailLength  = 0
  k = 0
  
  distFromHeadToTip = abs(math.sqrt((x-tailTip[0])**2 + (y-tailTip[1])**2))
  while (curTailLength < 1.5*distFromHeadToTip) and (k < len(points[0])-1):
    curTailLength = curTailLength + abs(math.sqrt((points[0,k]-points[0,k+1])**2 + (points[1,k]-points[1,k+1])**2))
    distToTip[k]  = abs(math.sqrt((points[0,k]-tailTip[0])**2 + (points[1,k]-tailTip[1])**2))
    curTailLengthTab[k] = curTailLength
    k = k + 1
  
  minDistToTip    = 1000000
  indMinDistToTip = 0
  for idx, dist in enumerate(distToTip):
    if dist < minDistToTip:
      minDistToTip = dist
      indMinDistToTip = idx
      
  hyperparameters["headEmbededParamTailDescentPixThreshStop"] = headEmbededParamTailDescentPixThreshStopInit
  
  return (curTailLengthTab[indMinDistToTip] )


def adjustHeadEmbededHyperparameters(hyperparameters, frame, headPosition, tailTip):
  
  dist = math.sqrt((headPosition[0] - tailTip[0])**2 + (headPosition[1] - tailTip[1])**2)
  factor = dist / 220
  
  if hyperparameters["headEmbededAutoSet_BackgroundExtractionOption"]:
    hyperparameters["headEmbededRemoveBack"] = 1
    hyperparameters["minPixelDiffForBackExtract"] = hyperparameters["headEmbededAutoSet_BackgroundExtractionOption"] # 7
    if hyperparameters["invertBlackWhiteOnImages"] == 0:
      hyperparameters["extractBackWhiteBackground"] = 0
    else:
      hyperparameters["extractBackWhiteBackground"] = 1
  
  if hyperparameters["headEmbededAutoSet_ExtendedDescentSearchOption"]:
    initialStepsTab = [10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50]
  else:
    initialStepsTab = [10, 13, 15]
  hyperparameters["step"] = [int(val*factor) for val in initialStepsTab]
  hyperparameters["step"] = list(dict.fromkeys(hyperparameters["step"])) # removes duplicate values
  if len(hyperparameters["step"]) == 0:
    hyperparameters["step"] = [2, 4]
    
  if hyperparameters["overwriteFirstStepValue"] or hyperparameters["overwriteLastStepValue"]:
    hyperparameters["overwriteFirstStepValue"] = int(hyperparameters["overwriteFirstStepValue"])
    hyperparameters["overwriteLastStepValue"]  = int(hyperparameters["overwriteLastStepValue"])
    if hyperparameters["overwriteLastStepValue"] <= hyperparameters["overwriteFirstStepValue"]:
      hyperparameters["overwriteLastStepValue"] = hyperparameters["overwriteFirstStepValue"] + 1
    hyperparameters["overwriteNbOfStepValues"] = hyperparameters["overwriteLastStepValue"] - hyperparameters["overwriteFirstStepValue"] + 1
    step = [hyperparameters["overwriteFirstStepValue"] + stepVal for stepVal in range(0, hyperparameters["overwriteNbOfStepValues"])]
    hyperparameters["step"] = step
  
  hyperparameters["headEmbededParamGaussianBlur"] = int(13 * factor)
  if hyperparameters["headEmbededParamGaussianBlur"] % 2 == 0:
    hyperparameters["headEmbededParamGaussianBlur"] = hyperparameters["headEmbededParamGaussianBlur"] + 1
  
  hyperparameters["addBlackCircleOfHalfDiamOnHeadForBoutDetect"] = int(70 * factor)
  
  hyperparameters["headEmbededTailTrackFindMaxDepthInitialMaxDepth"] = int(300*factor)
  
  hyperparameters["trackingPointSizeDisplay"] = int(5*factor) if int(5*factor) > 0 else 1 # CAN DO BETTER HERE !!!
  
  tailTipSurroundingDiameter = int(15 * factor)
  x = tailTip[0]
  y = tailTip[1]
  videoWidth  = hyperparameters["videoWidth"]
  videoHeight = hyperparameters["videoHeight"]
  if y-tailTipSurroundingDiameter < 0:
    ymin = 0
  else:
    ymin = y-tailTipSurroundingDiameter
  if y+tailTipSurroundingDiameter > videoHeight:
    ymax = videoHeight - 1
  else:
    ymax = y+tailTipSurroundingDiameter
  if x-tailTipSurroundingDiameter < 0:
    xmin = 0
  else:
    xmin = x-tailTipSurroundingDiameter
  if x+tailTipSurroundingDiameter > videoWidth:
    xmax = videoWidth - 1
  else:
    xmax = x+tailTipSurroundingDiameter
  hyperparameters["headEmbededParamTailDescentPixThreshStop"] = int(np.mean(frame[int(ymin):int(ymax), int(xmin):int(xmax)]))
  
  if hyperparameters["headEmbededParamTailDescentPixThreshStopOverwrite"] != -1:
    hyperparameters["headEmbededParamTailDescentPixThreshStop"] = hyperparameters["headEmbededParamTailDescentPixThreshStopOverwrite"]
  
  
  if False:
    print("headEmbededParamTailDescentPixThreshStop:", hyperparameters["headEmbededParamTailDescentPixThreshStop"])
    print("dist:", dist)
    print("step:", hyperparameters["step"])
    print("headEmbededParamGaussianBlur:", hyperparameters["headEmbededParamGaussianBlur"])
    print("trackingPointSizeDisplay:", hyperparameters["trackingPointSizeDisplay"])

  return hyperparameters



import numpy as np
import cv2

def headEmbededFrameSequential(cap, videoPath, frameNumber):
  
  debug = 0
  
  # cap = cv2.VideoCapture(videoPath)
  
  # cap.set(1, frameNumber)
  ret, frame = cap.read()
  
  while not(ret):
    print("WARNING: last frame had a value that was too high!")
    frameNumber = frameNumber - 1
    cap.set(1, frameNumber)
    ret, frame = cap.read()
  
  kernel = np.ones((8,8),np.float32)/25
  thres1  = cv2.filter2D(frame,-1,kernel)
  retval, thres1 = cv2.threshold(thres1, 80, 255, cv2.THRESH_BINARY)
  thres1 = 255 - thres1
  
  frame = 255 - frame
  
  if (debug):
    cv2.imshow('thres1', frame)
    cv2.waitKey(0)
    cv2.imshow('thres1', thres1)
    cv2.waitKey(0)
    
  frame  = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
  thres1 = cv2.cvtColor(thres1, cv2.COLOR_BGR2GRAY)
  
  return [frame, thres1]



import os
dir_path = os.path.dirname(os.path.realpath(__file__))
os.chdir(dir_path)
print("The data produced by ZebraZoom can be found in the folder: " + os.path.join(dir_path,'ZZoutput'))

import sys
sys.path.insert(1, './')
sys.path.insert(1, './code/')
sys.path.insert(1, './code/GUI/')
sys.path.insert(1, './code/getImage/')
sys.path.insert(1, './code/trackingFolder/')
sys.path.insert(1, './code/trackingFolder/headTrackingHeadingCalculationFolder/')
sys.path.insert(1, './code/trackingFolder/tailTrackingFunctionsFolder/')
sys.path.insert(1, './code/trackingFolder/tailTrackingFunctionsFolder/tailTrackingExtremityDetectFolder/')
sys.path.insert(1, './code/trackingFolder/tailTrackingFunctionsFolder/tailTrackingExtremityDetectFolder/findTailExtremeteFolder/')
sys.path.insert(1, './dataAnalysis/')
sys.path.insert(1, './dataAnalysis/dataanalysis/')
sys.path.insert(1, './dataAnalysis/datasetcreation/')


from vars import getGlobalVariables
globalVariables = getGlobalVariables()

from GUIAllPy import SampleApp

if __name__ == "__main__":
  app = SampleApp()
  app.mainloop()



import h5py
import numpy as np
import cv2
import math
import json
import sys
import matplotlib.pyplot as plt
from scipy import interpolate
from getForegroundImage import getForegroundImage
from headEmbededFrame import headEmbededFrame
from scipy.interpolate import UnivariateSpline
from numpy import linspace

def resampleSeqConstPtsPerArcLength(OrigBound, numTailPoints):
  
  n = len(OrigBound)
  distOrg = np.zeros(n)
  xOrg    = np.zeros(n)
  yOrg    = np.zeros(n)
  
  totDist = 0
  distOrg[0] = totDist
  xOrg[0] = OrigBound[0][0][0]
  yOrg[0] = OrigBound[0][0][1]
  
  for i in range(1, n):
    diff       = math.sqrt((OrigBound[i-1][0][0]-OrigBound[i][0][0])**2 + (OrigBound[i-1][0][1]-OrigBound[i][0][1])**2)
    totDist    = totDist + diff
    distOrg[i] = totDist
  
  uniDist = np.zeros(numTailPoints)
  uniX    = np.zeros(numTailPoints)
  uniY    = np.zeros(numTailPoints)
  
  for i in range(0, numTailPoints):
    uniDist[i] = totDist * (i/(numTailPoints-1))
  
  for i in range(1, n):
    xOrg[i] = OrigBound[i][0][0]
    yOrg[i] = OrigBound[i][0][1]
  
  uniX = np.interp(uniDist, distOrg, xOrg)
  uniY = np.interp(uniDist, distOrg, yOrg)
  
  output = np.zeros((numTailPoints, 2))
  for i in range(0, numTailPoints):
    output[i][0] = uniX[i]
    output[i][1] = uniY[i]
  
  return output



import sys

from findWells import findWells
from getBackground import getBackground
from getForegroundImage import getForegroundImage
from tracking import tracking
from extractParameters import extractParameters
from createSuperStruct import createSuperStruct
from createValidationVideo import createValidationVideo
from getHyperparameters import getHyperparameters

from sklearn.preprocessing import normalize
import multiprocessing as mp
from multiprocessing import Process
import shutil

import csv

import numpy as np

from getTailTipManual import findTailTipByUserInput
from getTailTipManual import findHeadPositionByUserInput
from headEmbededFrame import headEmbededFrame

def getTailExtremityFirstFrame(pathToVideo, videoName, videoExt, configFile, argv):
  
  videoName = videoName + '.' + videoExt
  
  videoPath = pathToVideo + videoName

  # Getting hyperparameters
  hyperparameters = getHyperparameters(configFile, videoName, videoPath, argv)
  
  frameNumber = hyperparameters["firstFrame"]
  videoNames = [videoName]
  
  if len(argv) > 5:
    for i in range(5, len(argv)):
      if argv[i] == "frameNumber":
        frameNumber = int(argv[i+1])
        break
      else:
        videoNames.append(argv[i])
  
  [frame, thresh1] = headEmbededFrame(videoPath, frameNumber)
  
  if hyperparameters["accentuateFrameForManualTailExtremityFind"]:
    quartileChose = 0.01
    lowVal  = int(np.quantile(frame, quartileChose))
    highVal = int(np.quantile(frame, 1 - quartileChose))
    frame[frame < lowVal]  = lowVal
    frame[frame > highVal] = highVal
    frame = frame - lowVal
    mult  = np.max(frame)
    frame = frame * (255/mult)
    frame = frame.astype(int)
    frame = (frame / np.linalg.norm(frame))*255
  
  tailTip  = findTailTipByUserInput(frame,hyperparameters)

  for videoN in videoNames:
  
    with open(pathToVideo+'/'+videoN+'.csv', mode='w') as employee_file:
      employee_writer = csv.writer(employee_file, delimiter=',', quotechar='"', quoting=csv.QUOTE_MINIMAL)
      employee_writer.writerow(tailTip)
      
    if hyperparameters["findHeadPositionByUserInput"]:
      headPosition = findHeadPositionByUserInput(frame)
      with open(pathToVideo+'/'+videoN+'HP.csv', mode='w') as employee_file:
        employee_writer = csv.writer(employee_file, delimiter=',', quotechar='"', quoting=csv.QUOTE_MINIMAL)
        employee_writer.writerow(headPosition)



import sys
# sys.path.insert(1, './code/')
# sys.path.insert(1, './code/tracking/')
# sys.path.insert(1, './code/getImage/')

from getForegroundImage import getForegroundImage
from createSuperStruct import createSuperStruct
from createValidationVideo import createValidationVideo
from getHyperparameters import getHyperparameters

import os
import shutil
import json
# import pdb

# Getting parameters
# if len(sys.argv) > 4:
  # pathToVideo = sys.argv[1]
  # videoName   = sys.argv[2]
  # videoExt    = sys.argv[3] # Ext means extension (avi, mp4, etc...)
  # configFile  = sys.argv[4]
  # argv        = sys.argv
# else:
  # configFile  = ""

def recreateSuperStruct(pathToVideo, videoName, videoExt, configFile, argv):
  
  # Getting parameters
  pathToVideo = sys.argv[1]
  videoName   = sys.argv[2]
  videoExt    = sys.argv[3] # Ext means extension (avi, mp4, etc...)
  configFile  = sys.argv[4]
  videoNameWithExt = videoName + '.' + videoExt

  # Getting hyperparameters
  hyperparameters = getHyperparameters(configFile, videoNameWithExt, pathToVideo + videoNameWithExt, argv)
    
  # Saving the configuration file used
  shutil.copyfile(configFile, hyperparameters["outputFolder"] + videoName + '/configUsed.txt')
  
  
  with open(hyperparameters["outputFolder"] + videoName + '/results_' + videoName + '.txt') as inires:
    iniRes = json.load(inires)
  
  # Getting well positions
  wellPositions = iniRes["wellPositions"]
  
  # Sorting wells after the end of the parrallelized calls end
  dataPerWell = iniRes["wellPoissMouv"]
  n = len(dataPerWell)
  for i in range(0,n):
    dataPerWell[i] = dataPerWell[i][0]
  
  # Creating super structure
  superStruct = createSuperStruct(dataPerWell, wellPositions, hyperparameters)
  if (hyperparameters["freqAlgoPosFollow"] != 0):
    print("superStruct created")
    
  if hyperparameters["createValidationVideo"]:
    # Creating validation video
    infoFrame = createValidationVideo(pathToVideo + videoNameWithExt, superStruct, hyperparameters)


# if __name__ == '__main__':

  # __spec__ = "ModuleSpec(name='builtins', loader=<class '_frozen_importlib.BuiltinImporter'>)"
  
  # recreateSuperStruct(pathToVideo, videoName, videoExt, configFile, argv)



import h5py
import numpy as np
import cv2
import math
import json
import sys
import matplotlib.pyplot as plt
from scipy import interpolate
from getForegroundImage import getForegroundImage
from headEmbededFrame import headEmbededFrame
from scipy.interpolate import UnivariateSpline
from numpy import linspace

from resampleSeqConstPtsPerArcLength import resampleSeqConstPtsPerArcLength

def fillTailRanges(tailRange1,tailRange2,fillSecond,i,MostCurvyIndex):
  if (i == MostCurvyIndex):
    fillSecond = 1
  if fillSecond == 0:
    tailRange1.append(i)
  else:
    tailRange2.append(i)
  return [tailRange1,tailRange2,fillSecond]

def getMidline(bord1, bord2, MostCurvyIndex, boundary, dst, taille, distance2, debug, hyperparameters):

  nbTailPoints = taille
  output = np.zeros((1, 0, 2))

  numTailPoints = 10
  minTailSize = 20
  maxTailSize = 60
  trackingPointSizeDisplay = 1

  OrigBoundA = []
  OrigBoundB = []
  
  if (bord2 < bord1):
    temp  = bord2
    bord2 = bord1
    bord1 = temp
    
  max1 = distance2[bord2] - distance2[bord1] 
  max2 = (distance2[bord1] - distance2[0])  + (distance2[len(boundary)] - distance2[bord2])
  
  tailRangeA = []
  tailRangeB = []
  fillSecond = 0
  if (max1 > max2):
    for i in range(bord1, bord2):
      [tailRangeA,tailRangeB,fillSecond] = fillTailRanges(tailRangeA,tailRangeB,fillSecond,i,MostCurvyIndex)
  else:
    for i in range(bord2, len(boundary)):
      [tailRangeA,tailRangeB,fillSecond] = fillTailRanges(tailRangeA,tailRangeB,fillSecond,i,MostCurvyIndex)
    for i in range(0, bord1):
      [tailRangeA,tailRangeB,fillSecond] = fillTailRanges(tailRangeA,tailRangeB,fillSecond,i,MostCurvyIndex)
  
  OrigBoundA = boundary[tailRangeA]
  OrigBoundB = boundary[tailRangeB]
  
  if ((bord1!=bord2) and (bord1!=MostCurvyIndex) and (bord2!=MostCurvyIndex) and not((bord1==1) and (bord2==1) and (MostCurvyIndex==1)) and (len(OrigBoundA)>1) and (len(OrigBoundB)>1)):

    NBoundA = resampleSeqConstPtsPerArcLength(OrigBoundA, numTailPoints)
    NBoundB = resampleSeqConstPtsPerArcLength(OrigBoundB, numTailPoints)

    # calculates length of the tail
    TotalDist = 0
    for i in range(1, taille):
      Pt  = NBoundB[i % taille]
      Pt2 = NBoundA[taille - i]
      x = (Pt[0]+Pt2[0]) / 2
      y = (Pt[1]+Pt2[1]) / 2
      if i > 1:
        TotalDist = TotalDist + math.sqrt((x-xAvant)*(x-xAvant)+(y-yAvant)*(y-yAvant))
      xAvant = x
      yAvant = y
      
    if ((TotalDist<hyperparameters["minTailSize"]) or (TotalDist>hyperparameters["maxTailSize"])):
    
      if (debug):
        print("innapropriate tail size! TailDist: ", TotalDist, " ; but minTailSize is ", minTailSize, " and maxTailSize is ", maxTailSize)
        
    else:
      
      Tail = boundary[MostCurvyIndex][0]
      
      point = np.array([Tail[0], Tail[1]])
      output = np.insert(output, 0, point, axis=1)
      
      for i in range(1, taille):
        Pt  = NBoundB[i % taille]
        Pt2 = NBoundA[taille - i]
        point = np.array([(Pt[0]+Pt2[0])/2, (Pt[1]+Pt2[1])/2])
        output = np.insert(output, 0, point, axis=1)
      
      i = taille-2
      if i >= 1:
        Pt =  NBoundB[i % taille]
        Pt2 = NBoundA[taille-i]
        ClosestPoint = [ (Pt[0]+Pt2[0])/2 , (Pt[1]+Pt2[1])/2 ]
      else:
        ClosestPoint = [-200, -200]
  
  return output



import cv2
import cvui
import numpy as np
import pickle
import math

def initializeAdjustHyperparametersWindows(WINDOW_NAME):
  cv2.destroyAllWindows()
  WINDOW_NAME_CTRL = "Adjust Parameters."
  cvui.init(WINDOW_NAME)
  cv2.moveWindow(WINDOW_NAME, 0, 0)
  cvui.init(WINDOW_NAME_CTRL)
  cv2.moveWindow(WINDOW_NAME_CTRL, 0, 300)


def printStuffOnCtrlImg(frameCtrl, frameNum, x, y, l, minn, maxx, name, hint):
  cvui.text(frameCtrl, x, y, name)
  
  status = cvui.iarea(x, y, 350, 60)
  if status == cvui.OVER:
    cvui.text(frameCtrl, 4, 15, hint)
    
  if name != "Frame number":
    if (frameNum[0] - minn) > (maxx - minn) * 0.9:
      maxx = minn + (frameNum[0] - minn) * 1.25
    if maxx - minn > 255:
      if frameNum[0] < minn + (maxx - minn) * 0.1:
        maxx = minn + (maxx - minn) * 0.9

  cvui.trackbar(frameCtrl, x,      y+10, l, frameNum, minn, maxx)
  cvui.counter(frameCtrl,  x+l+10, y+20,    frameNum)
  
  return [minn, maxx]


def adjustHyperparameters(l, hyperparameters, hyperparametersListNames, frameToShow, WINDOW_NAME, organizationTab):
  
  WINDOW_NAME_CTRL = "Adjust Parameters."
  frameNum            = [l]
  hyperparametersList = [[hyperparameters[name]] for name in hyperparametersListNames]
  frameCtrlLengthY = math.ceil((len(hyperparametersListNames) + 2)/2) * 70 + 20
  
  buttonclicked = False
  while frameNum[0] == l and hyperparametersList == [[hyperparameters[name]] for name in hyperparametersListNames] and not(buttonclicked):
    
    frameCtrl = np.full((frameCtrlLengthY, 1100), 100).astype('uint8')
    printStuffOnCtrlImg(frameCtrl, frameNum, 1, 35, 350, hyperparameters["firstFrame"], hyperparameters["lastFrame"], "Frame number", "You can also go through the video with the keys a or 4 (backward); d or 6 (forward); f or g (fast backward); h or j (fast forward)")
    for idx, hyperParamCurVal in enumerate(hyperparametersList):
      [minn, maxx] = printStuffOnCtrlImg(frameCtrl, hyperParamCurVal, organizationTab[idx][0], organizationTab[idx][1], organizationTab[idx][2], organizationTab[idx][3], organizationTab[idx][4], hyperparametersListNames[idx], organizationTab[idx][5])
      organizationTab[idx][3] = minn
      organizationTab[idx][4] = maxx
    
    buttonclicked = cvui.button(frameCtrl, organizationTab[len(organizationTab)-1][0], organizationTab[len(organizationTab)-1][1], "Ok, done!")
    # cvui.text(frameCtrl, 100, 245, 'Warning: for some of the "overwrite" parameters, you will need to change the initial value for the "overwrite" to take effect.')
    cvui.imshow(WINDOW_NAME, frameToShow)
    cvui.imshow(WINDOW_NAME_CTRL, frameCtrl)
    r = cv2.waitKey(20)
    if (r == 54) or (r == 100) or (r == 0):
      frameNum[0] = frameNum[0] + 1
    elif (r == 52) or (r == 97) or (r == 113):
      frameNum[0] = frameNum[0] - 1
  l = int(frameNum[0])
  for idx, hyperParamCurVal in enumerate(hyperparametersList):
    hyperparameters[hyperparametersListNames[idx]] = hyperParamCurVal[0]
  if buttonclicked:
    newhyperparameters = {}
    for idx, hyperparameterName in enumerate(hyperparametersListNames):
      newhyperparameters[hyperparameterName] = hyperparameters[hyperparameterName]
    pickle.dump(newhyperparameters, open('newhyperparameters', 'wb'))
    cv2.destroyAllWindows()
    raise ValueError

  return [l, hyperparameters, organizationTab]


def getDetectMouvRawVideosParamsForHyperParamAdjusts(img, res, l, totDiff, hyperparameters):
  hyperparametersListNames = ["frameGapComparision", "thresForDetectMovementWithRawVideo", "halfDiameterRoiBoutDetect", "minNbPixelForDetectMovementWithRawVideo"]
  marginX = 30
  organizationTab = [\
  [470,  marginX + 5,  350,  0, 15,  "Increase if small movements are not detected. An increase that's too big could lead to a detection of bout that's too early however."],
  [1,    marginX + 71, 350,  0, 255, "Increase if too much movement is being detected."],
  [470,  marginX + 71, 350,  0, 500, "Controls the size of the images on which pixel change from image to the next are counted."],
  [1,   marginX + 137, 350,  0, 50,  "Increase if too much movement is being detected."],
  [470, marginX + 137,  -1, -1, -1,  "Click here once the red dot appears on enough images where movement is occurring."]]
  WINDOW_NAME = "Bouts Detection"
  frameToShow = np.concatenate((img, res),axis=1)
  frameToShow = cv2.cvtColor(frameToShow,cv2.COLOR_GRAY2RGB)
  frameToShow = cv2.putText(frameToShow, str(l), (1, 20), cv2.FONT_HERSHEY_SIMPLEX, 1, (0,255,0), 3)
  if totDiff > hyperparameters["minNbPixelForDetectMovementWithRawVideo"]:
    cv2.circle(frameToShow, (20, 40), 20, (0,0,255), -1)
  return [hyperparametersListNames, frameToShow, WINDOW_NAME, organizationTab]


def getHeadEmbededTrackingParamsForHyperParamAdjusts(nbTailPoints, i, firstFrame, x, y, output, outputHeading, frame, frame2, hyperparameters):
  
  # hyperparametersListNames = ["headEmbededAutoSet_BackgroundExtractionOption", "overwriteFirstStepValue", "overwriteLastStepValue", "overwriteNbOfStepValues", "headEmbededParamTailDescentPixThreshStopOverwrite", "authorizedRelativeLengthTailEnd"]
  hyperparametersListNames = ["headEmbededAutoSet_BackgroundExtractionOption", "overwriteFirstStepValue", "overwriteLastStepValue", "headEmbededParamTailDescentPixThreshStopOverwrite", "authorizedRelativeLengthTailEnd"]
  marginX = 30
  organizationTab = [\
  [470,   marginX + 5, 350,  0,  20, "Transforms non-background pixels to black. Can be useful when the tail isn't very different from the background."],
  [1,    marginX + 71, 350,  0,  80, "Increase this to avoid having the tracking points go on the head instead of the tail."],
  [470,  marginX + 71, 350,  1,  80, 'Increase this if the tail tracking is getting off track "mid-tail". Decrease if the tail tracking is going too far (further than the tip).'],
  # [1,   marginX + 137, 350,  1,  50, "This is set automatically when you change either overwriteFirstStepValue or overwriteLastStepValue. Decrease to make the tracking faster."],
  [1,   marginX + 137, 350,  0, 255, "Decrease if the tail tracking is going too far (further than the tip of the tail). Increase if the tail if not going far enough (stops before the tip)."],
  [470, marginX + 137, 350,  0, 1, 'Relative length along the "normal lenght" of the tail where the tracking is "allowed" to stop. Decrease if the tail becomes invisible "mid-tail".'],
  [1, marginX + 208,  -1, -1,  -1, "Click here if you're done adjusting these parameters."],]
  WINDOW_NAME = "Tracking"
  
  # frame2 = np.concatenate((frame2, frame),axis=1)
  
  frame2 = cv2.cvtColor(frame2,cv2.COLOR_GRAY2RGB)
  for k in range(0, hyperparameters["nbAnimalsPerWell"]):
    for j in range(0, nbTailPoints):
      x = int(output[k, i-firstFrame][j][0])
      y = int(output[k, i-firstFrame][j][1])
      cv2.circle(frame2, (x, y), 1, (0, 255, 0),   -1)
    x = int(output[k, i-firstFrame][nbTailPoints-1][0])
    y = int(output[k, i-firstFrame][nbTailPoints-1][1])
    cv2.circle(frame2, (x, y), 2, (0, 0, 255),   -1)
    x = output[k, i-firstFrame][0][0]
    y = output[k, i-firstFrame][0][1]
  
  return [hyperparametersListNames, frame2, WINDOW_NAME, organizationTab]


def getFreelySwimTrackingParamsForHyperParamAdjusts(nbTailPoints, i, firstFrame, x, y, output, outputHeading, frame, frame2, hyperparameters):
  
  hyperparametersListNames = ["minPixelDiffForBackExtract", "maxAreaBody", "minTailSize", "maxTailSize"]
  marginX = 30
  organizationTab = [\
  [470,   marginX + 5, 350,  0,  20, "Increase this if some of the background is not completely white. Decrease if you can't see all of the fish. "],
  [1,    marginX + 71, 350,  0,  20, "Try increasing this if no tracking is showing."],
  [470,  marginX + 71, 350,  0,  20, "Try increasing this if no tracking is showing."],
  [1,   marginX + 137, 350,  0,  20, "Try decreasing this if no tracking is showing."],
  [470, marginX + 137, -1,  -1, "Click here if you're done adjusting these parameters."]]
  
  # [1,    marginX + 71, 350,  0,  20, "minArea"],
  # [470,  marginX + 71, 350,  0,  20, "maxArea"],
  # [1,   marginX + 137, 350,  0,  20, "minAreaBody"],
  # [470, marginX + 137, 350,  0,  20, "maxAreaBody"],
  # [1,   marginX + 208, -1,  -1, "Click here if you're done adjusting these parameters."],]
  
  WINDOW_NAME = "Tracking"
  
  frame2 = cv2.cvtColor(frame2,cv2.COLOR_GRAY2RGB)
  for k in range(0, hyperparameters["nbAnimalsPerWell"]):
    for j in range(0, nbTailPoints):
      x = int(output[k, i-firstFrame][j][0])
      y = int(output[k, i-firstFrame][j][1])
      cv2.circle(frame2, (x, y), 1, (0, 255, 0),   -1)
    x = int(output[k, i-firstFrame][nbTailPoints-1][0])
    y = int(output[k, i-firstFrame][nbTailPoints-1][1])
    cv2.circle(frame2, (x, y), 2, (0, 0, 255),   -1)
    x = output[k, i-firstFrame][0][0]
    y = output[k, i-firstFrame][0][1]
  
  return [hyperparametersListNames, frame2, WINDOW_NAME, organizationTab]



import tkinter as tk
from tkinter import font  as tkfont
from tkinter import filedialog
from tkinter import ttk
from tkinter import *
import re
import os
import json
import subprocess
import matplotlib
matplotlib.use("TkAgg")
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.figure import Figure
import math
import scipy.io as sio
from readValidationVideo import readValidationVideo
from vars import getGlobalVariables
globalVariables = getGlobalVariables()

import sys
sys.path.insert(1, './')
sys.path.insert(1, './code/')
sys.path.insert(1, './code/getImage/')
sys.path.insert(1, './code/tracking/')
sys.path.insert(1, './code/tracking/tailTrackingExtremityDetect/')
sys.path.insert(1, './code/tracking/tailTrackingExtremityDetect/findTailExtremete/')
from mainZZ import mainZZ
from getTailExtremityFirstFrame import getTailExtremityFirstFrame
import popUpAlgoFollow

LARGE_FONT= ("Verdana", 12)

def chooseVideoToAnalyze(self, justExtractParams, noValidationVideo):
    
    if globalVariables["mac"]:
        tk.videoName =  filedialog.askopenfilename(initialdir = os.path.expanduser("~"),title = "Select file")
    else:
        tk.videoName =  filedialog.askopenfilename(initialdir = os.path.expanduser("~"),title = "Select file",filetypes = (("video","*.*"),("all files","*.*")))
    tk.folderName = ''
    tk.headEmbedded = 0
    
    tk.justExtractParams = int(justExtractParams)
    tk.noValidationVideo = int(noValidationVideo)
    
    self.show_frame("ConfigFilePromp")

def chooseFolderToAnalyze(self, justExtractParams, noValidationVideo):
    tk.folderName =  filedialog.askdirectory(initialdir = os.path.expanduser("~"),title = "Select folder")
    tk.headEmbedded = 0
    tk.justExtractParams = int(justExtractParams)
    tk.noValidationVideo = int(noValidationVideo)
    self.show_frame("ConfigFilePromp")
    
def chooseFolderForTailExtremityHE(self):
    tk.folderName =  filedialog.askdirectory(initialdir = os.path.expanduser("~"),title = "Select folder")
    tk.headEmbedded = 1
    self.show_frame("ConfigFilePromp")

def chooseConfigFile(self):
    if globalVariables["mac"]:
        tk.configFile =  filedialog.askopenfilename(initialdir = "./configuration/",title = "Select file")
    else:
        tk.configFile =  filedialog.askopenfilename(initialdir = "./configuration/",title = "Select file",filetypes = (("json files","*.json"),("all files","*.*")))
    if globalVariables["mac"] or globalVariables["lin"]:
        self.show_frame("Patience")
    else:
        self.launchZebraZoom()

def launchZebraZoom(self):
  # __spec__ = "ModuleSpec(name='builtins', loader=<class '_frozen_importlib.BuiltinImporter'>)"
  last = 0
  allVideos = []
  
  if len(tk.folderName):
    for name in os.listdir(tk.folderName):
      if len(name) > 3:
        ext = name[len(name)-3:len(name)]
        if (ext == 'mp4') or (ext == 'avi'):
          allVideos.append(tk.folderName+'/'+name)
  else:
    allVideos = [tk.videoName]
  
  # if tk.headEmbedded == 0:
    # popUpAlgoFollow.initialise()
  
  for idx, text in enumerate(allVideos):
    for m in re.finditer('/', text):
      last = m.start()
    path        = text[:last+1]
    nameWithExt = text[last+1:]
    pointPos = nameWithExt.find('.')
    name     = nameWithExt[:pointPos]
    videoExt = nameWithExt[pointPos+1:]
    # closePopUpWindowAtTheEnd = (idx == len(allVideos)-1)
    
    if tk.headEmbedded == 0:
      tabParams = ["mainZZ", path, name, videoExt, tk.configFile, "freqAlgoPosFollow", 100, "popUpAlgoFollow", 1]
      if tk.justExtractParams == 1:
        tabParams = tabParams + ["reloadWellPositions", 1, "reloadBackground", 1, "debugPauseBetweenTrackAndParamExtract", "justExtractParamFromPreviousTrackData"]
      if tk.noValidationVideo == 1:
          tabParams = tabParams + ["createValidationVideo", 0]
      mainZZ(path, name, videoExt, tk.configFile, tabParams)
    else:
      getTailExtremityFirstFrame(path, name, videoExt, tk.configFile, [])
  
  self.show_frame("ZZoutro")


def showValidationVideo(self, numWell, zoom, deb):
    # filepath = 'ZZoutput/'+self.currentResultFolder+'/pathToVideo.txt'
    # with open(filepath) as fp:
       # videoPath = fp.readline()
    # videoPath = videoPath[:len(videoPath)-1]
    
    filepath = './ZZoutput/'+self.currentResultFolder+'/pathToVideo.txt'
    if os.path.exists(filepath):
        with open(filepath) as fp:
           videoPath = fp.readline()
        videoPath = videoPath[:len(videoPath)-1]
    else:
        videoPath = ""
    
    readValidationVideo(videoPath, self.currentResultFolder,'.txt',int(numWell),int(zoom),int(deb))
    

def exploreResultFolder(self, currentResultFolder):
    self.currentResultFolder = currentResultFolder
    self.superstructmodified = 0
    self.justEnteredViewParameter = 1
    self.printSomeResults(0, 0, 0)
    
    
def printNextResults(self, numWell, numPoiss, numMouv, nbWells, nbPoiss, nbMouv):

    numWell  = int(numWell)
    numPoiss = int(numPoiss)
    numMouv  = int(numMouv)
    nbWells  = int(nbWells)
    nbPoiss  = int(nbPoiss)
    nbMouv   = int(nbMouv)
    
    if numMouv + 1 >= nbMouv:
        if numPoiss + 1 >= nbPoiss:
            if numWell + 1 < nbWells:
              numWell  = numWell + 1
              numPoiss = 0
              numMouv  = 0
        else:
            numPoiss = numPoiss + 1
            numMouv  = 0
    else:
        numMouv = numMouv + 1

    self.printSomeResults(numWell, numPoiss, numMouv)


def printPreviousResults(self, numWell, numPoiss, numMouv, nbWells, nbPoiss, nbMouv):

    numWell  = int(numWell)
    numPoiss = int(numPoiss)
    numMouv  = int(numMouv)
    nbWells  = int(nbWells)
    nbPoiss  = int(nbPoiss)
    nbMouv   = int(nbMouv)
    
    if numMouv - 1 < 0:
        if numPoiss - 1 < 0:
            if numWell - 1 >= 0:
                numWell = numWell - 1
                numPoiss = 100000
                numMouv = 10000000
        else:
            numPoiss = numPoiss - 1
            numMouv  = 0
    else:
        numMouv = numMouv - 1

    self.printSomeResults(numWell, numPoiss, numMouv)


def flagMove(self, numWell, numPoiss, numMouv):

    self.superstructmodified = 1

    name = self.currentResultFolder

    reference = './ZZoutput/'+name+'/results_'+name+'.txt'

    dataRef = self.dataRef
    
    if "flag" in dataRef["wellPoissMouv"][int(numWell)][int(numPoiss)][int(numMouv)]:
      dataRef["wellPoissMouv"][int(numWell)][int(numPoiss)][int(numMouv)]["flag"] = int(not(dataRef["wellPoissMouv"][int(numWell)][int(numPoiss)][int(numMouv)]["flag"]));
    else:
      dataRef["wellPoissMouv"][int(numWell)][int(numPoiss)][int(numMouv)]["flag"] = 1;
    
    self.dataRef = dataRef
    
    self.printSomeResults(numWell, numPoiss, numMouv)


def saveSuperStruct(self, numWell, numPoiss, numMouv):
    
    self.superstructmodified = 0
    
    name = self.currentResultFolder
    dataRef = self.dataRef
    
    reference = './ZZoutput/'+name+'/results_'+name+'.txt'
    
    with open(reference,'w') as out:
       json.dump(dataRef, out)
    
    # sio.savemat('../ZZoutput/'+name+'/results_'+name+'.mat', {'videoDataResults':dataRef})
    
    # f=open("lance.m","w+")
    # f.write("cd ZebraZoom\n")
    # f.write("saveToMatlabFromGUI('../ZZoutput/" + name + "/results_" + name + ".mat')\n")
    # f.write("cd ..\n")
    # f.write("quit\n")
    
    # f2=open("lance.sh","w+")
    # f2.write("matlab -r lance -nodesktop -nojvm -nosplash\n")
    
    # os.startfile('lance.sh')
    
    self.dataRef = dataRef
    
    self.printSomeResults(numWell, numPoiss, numMouv)


def openConfigurationFileFolder(self, homeDirectory):
  # dir_path = os.path.dirname(os.path.realpath(__file__))
  dir_path = os.path.join(homeDirectory,'configuration')
  if sys.platform == "win32":
    os.startfile(dir_path)
  else:
    opener ="open" if sys.platform == "darwin" else "xdg-open"
    subprocess.call([opener, dir_path])


def openZZOutputFolder(self, homeDirectory):
  # dir_path = os.path.dirname(os.path.realpath(__file__))
  dir_path = os.path.join(homeDirectory,'ZZoutput')
  if sys.platform == "win32":
    os.startfile(dir_path)
  else:
    opener ="open" if sys.platform == "darwin" else "xdg-open"
    subprocess.call([opener, dir_path])  



import h5py
import numpy as np
import cv2
import math
import json
import sys
import matplotlib.pyplot as plt
from scipy import interpolate
from getForegroundImage import getForegroundImage
from headEmbededFrame import headEmbededFrame
from scipy.interpolate import UnivariateSpline
from numpy import linspace

def findBodyContour(headPosition, hyperparameters, thresh1):
  x = headPosition[0]
  y = headPosition[1]
  cx = 0
  cy = 0
  takeTheHeadClosestToTheCenter = 1
  minAreaCur = hyperparameters["minAreaBody"]
  maxAreaCur = hyperparameters["maxAreaBody"]
  bodyContour = 0
  while (cx == 0) and (minAreaCur > -200):
    if hyperparameters["findContourPrecision"] == "CHAIN_APPROX_SIMPLE":
      contourPrecision = cv2.CHAIN_APPROX_SIMPLE
    else: # hyperparameters["findContourPrecision"] == "CHAIN_APPROX_NONE"
      contourPrecision = cv2.CHAIN_APPROX_NONE
    contours, hierarchy = cv2.findContours(thresh1, cv2.RETR_TREE, contourPrecision)
    for contour in contours:
      area = cv2.contourArea(contour)
      if (area > minAreaCur) and (area < maxAreaCur):
        M = cv2.moments(contour)
        if takeTheHeadClosestToTheCenter == 0:
          if M['m00']:
            cx = int(M['m10']/M['m00'])
            cy = int(M['m01']/M['m00'])
            bodyContour = contour
          else:
            cx = 0
            cy = 0
        else:
          if M['m00']:
            cxNew = int(M['m10']/M['m00'])
            cyNew = int(M['m01']/M['m00'])
          else:
            cx = 0
            cy = 0
          distToCenterNew = math.sqrt((cxNew-x)**2 + (cyNew-y)**2)
          distToCenter    = math.sqrt((cx-x)**2    + (cy-y)**2)
          if distToCenterNew < distToCenter:
            cx = cxNew
            cy = cyNew
            bodyContour = contour
    minAreaCur = minAreaCur - 100
    maxAreaCur = maxAreaCur + 100
  return bodyContour



import h5py
import numpy as np
import cv2
import popUpAlgoFollow
import math
import os

from getImages import getImages
from headTrackingHeadingCalculation import headTrackingHeadingCalculation
from debugTracking import debugTracking
from tailTracking import tailTracking
from blackFramesDetection import getThresForBlackFrame, savingBlackFrames
from getTailTipManual import getHeadPositionByFileSaved, findTailTipByUserInput, getTailTipByFileSaved, findHeadPositionByUserInput

from headEmbededFrame import headEmbededFrame
from headEmbededFrameBackExtract import headEmbededFrameBackExtract

from adjustHyperparameters import initializeAdjustHyperparametersWindows, adjustHyperparameters, getHeadEmbededTrackingParamsForHyperParamAdjusts, getFreelySwimTrackingParamsForHyperParamAdjusts
from headEmbededTailTracking import adjustHeadEmbededHyperparameters

from headEmbededTailTracking import headEmbededTailTrackFindMaxDepth
from headEmbededTailTrackingTeresaNicolson import headEmbededTailTrackFindMaxDepthTeresaNicolson, headEmbededTailTrackingTeresaNicolson
from centerOfMassTailTracking import centerOfMassTailTrackFindMaxDepth

def tracking(videoPath,background,wellNumber,wellPositions,hyperparameters,videoName):

  firstFrame = hyperparameters["firstFrame"]
  if hyperparameters["firstFrameForTracking"] != -1:
    firstFrame = hyperparameters["firstFrameForTracking"]
  lastFrame = hyperparameters["lastFrame"]
  
  if (hyperparameters["freqAlgoPosFollow"] != 0):
    print("lastFrame:",lastFrame)

  nbTailPoints = hyperparameters["nbTailPoints"]
  thetaDiffAccept = 1.2 # 0.5 for the head embedded maybe
  maxDepth = 0
  headPosition = []
  tailTip = []
  
  cap = cv2.VideoCapture(videoPath)
  if (cap.isOpened()== False): 
    print("Error opening video stream or file")
  frame_width  = int(cap.get(3))
  frame_height = int(cap.get(4))
  
  heading = -1
  if hyperparameters["headEmbeded"]:
    heading = 0.7
    
  output = np.zeros((hyperparameters["nbAnimalsPerWell"], lastFrame-firstFrame+1, nbTailPoints, 2))
  outputHeading = np.zeros((hyperparameters["nbAnimalsPerWell"], lastFrame-firstFrame+1))
  
  threshForBlackFrames = getThresForBlackFrame(hyperparameters, videoPath) # For headEmbededTeresaNicolson 
  cap.set(1, firstFrame)
  
  if (hyperparameters["headEmbeded"] == 1):
    # Getting images
    if hyperparameters["headEmbededRemoveBack"] == 0:
      [frame, thresh1] = headEmbededFrame(videoPath, firstFrame)
    else:
      [frame, thresh1] = headEmbededFrameBackExtract(videoPath, background, hyperparameters, firstFrame)
    gray = frame.copy()
    oppHeading = (heading + math.pi) % (2 * math.pi)
    # Getting head and tailtip positions
    if os.path.exists(videoPath+'HP.csv'):
      headPosition = getHeadPositionByFileSaved(videoPath)
    else:
      if hyperparameters["findHeadPositionByUserInput"]:
        headPosition = findHeadPositionByUserInput(frame)
      else:
        [frame, gray, thresh1, blur, thresh2, frame2] = getImages(hyperparameters, cap, videoPath, firstFrame, background, wellNumber, wellPositions)
        cap.set(1, firstFrame)
        [outputHeading, output, heading, headPosition, x, y, lastFirstTheta] = headTrackingHeadingCalculation(hyperparameters, firstFrame, firstFrame, blur, thresh1, thresh2, gray, hyperparameters["erodeSize"], frame_width, frame_height, outputHeading, output, headPosition, wellPositions[wellNumber]["lengthX"])
    if os.path.exists(videoPath+'.csv'):
      tailTip  = getTailTipByFileSaved(hyperparameters,videoPath)
    else:
      tailTip  = findTailTipByUserInput(frame,hyperparameters)
    if hyperparameters["automaticallySetSomeOfTheHeadEmbededHyperparameters"] == 1:
      hyperparameters = adjustHeadEmbededHyperparameters(hyperparameters, frame, headPosition, tailTip)
    # Getting max depth
    if hyperparameters["headEmbededTeresaNicolson"] == 1:
      maxDepth = headEmbededTailTrackFindMaxDepthTeresaNicolson(headPosition,nbTailPoints,firstFrame,headPosition[0],headPosition[1],thresh1,frame,hyperparameters,oppHeading,tailTip)
    else:
      if hyperparameters["centerOfMassTailTracking"] == 0:
        maxDepth = headEmbededTailTrackFindMaxDepth(headPosition,nbTailPoints,firstFrame,headPosition[0],headPosition[1],thresh1,frame,hyperparameters,oppHeading,tailTip)
      else:
        maxDepth = centerOfMassTailTrackFindMaxDepth(headPosition,nbTailPoints,firstFrame,headPosition[0],headPosition[1],thresh1,frame,hyperparameters,oppHeading,tailTip)
  
  if hyperparameters["adjustHeadEmbededTracking"] == 1 or hyperparameters["adjustFreelySwimTracking"] == 1:
    initializeAdjustHyperparametersWindows("Tracking")
  organizationTabCur = []
  
  # Performing the tracking on each frame
  i = firstFrame
  while (i < lastFrame+1):
    
    if (hyperparameters["freqAlgoPosFollow"] != 0) and (i % hyperparameters["freqAlgoPosFollow"] == 0):
      print("Tracking: wellNumber:",wellNumber," ; frame:",i)
      if hyperparameters["popUpAlgoFollow"]:
        popUpAlgoFollow.prepend("Tracking: wellNumber:" + str(wellNumber) + " ; frame:" + str(i))
    if hyperparameters["debugTracking"]:
      print("frame:",i)
    # Get images for frame i
    [frame, gray, thresh1, blur, thresh2, frame2] = getImages(hyperparameters, cap, videoPath, i, background, wellNumber, wellPositions)
    # Head tracking and heading calculation
    [outputHeading, output, heading, headPosition, x, y, lastFirstTheta] = headTrackingHeadingCalculation(hyperparameters, firstFrame, i, blur, thresh1, thresh2, gray, hyperparameters["erodeSize"], frame_width, frame_height, outputHeading, output, headPosition, wellPositions[wellNumber]["lengthX"])
    # Tail tracking for frame i
    if hyperparameters["nbAnimalsPerWell"] == 1 and hyperparameters["trackTail"] == 1 :
      animalId = 0
      
      [output, maxDepth, tailTip, headPosition] = tailTracking(animalId, i, firstFrame, heading, videoPath, x, y, headPosition, frame, hyperparameters, thresh1, nbTailPoints, threshForBlackFrames, thetaDiffAccept, output, lastFirstTheta, maxDepth, tailTip)
    
    # Debug functions
    debugTracking(nbTailPoints, i, firstFrame, x, y, output, outputHeading, frame2, hyperparameters)
    
    if hyperparameters["adjustHeadEmbededTracking"] == 1:
      [hyperparametersListNames, frameToShow, WINDOW_NAME, organizationTab] = getHeadEmbededTrackingParamsForHyperParamAdjusts(nbTailPoints, i, firstFrame, x, y, output, outputHeading, frame, frame2, hyperparameters)
      if len(organizationTabCur) == 0:
        organizationTabCur = organizationTab
      [i, hyperparameters, organizationTabCur] = adjustHyperparameters(i, hyperparameters, hyperparametersListNames, frameToShow, WINDOW_NAME, organizationTabCur)
      hyperparameters = adjustHeadEmbededHyperparameters(hyperparameters, frame, headPosition, tailTip)
    elif hyperparameters["adjustFreelySwimTracking"] == 1:
      [hyperparametersListNames, frameToShow, WINDOW_NAME, organizationTab] = getFreelySwimTrackingParamsForHyperParamAdjusts(nbTailPoints, i, firstFrame, x, y, output, outputHeading, frame, frame2, hyperparameters)
      if len(organizationTabCur) == 0:
        organizationTabCur = organizationTab
      [i, hyperparameters, organizationTabCur] = adjustHyperparameters(i, hyperparameters, hyperparametersListNames, frameToShow, WINDOW_NAME, organizationTabCur)
    else:
      i = i + 1
  
  savingBlackFrames(hyperparameters, videoName, output)
  
  print("Tracking done for well", wellNumber)
  if hyperparameters["popUpAlgoFollow"]:
    popUpAlgoFollow.prepend("Tracking done for well "+ str(wellNumber))
  
  return [output, outputHeading, headPosition, tailTip]



import h5py
import numpy as np
import cv2
import math
import json
import sys
import matplotlib.pyplot as plt
from scipy import interpolate
from getForegroundImage import getForegroundImage
from headEmbededFrame import headEmbededFrame
from scipy.interpolate import UnivariateSpline
from numpy import linspace

def findTheTwoSides(headPosition, bodyContour, dst):

  aaa = headPosition[0]
  bbb = headPosition[1]
  minNbPointsBetweenTwoBordersIndexes     = 3
  findSecondTailBasisVectorCreationFactor = 1
  dist = 0
  min  = 10000000
  coolIndex  = 0
  coolIndex2 = 0
  k = findSecondTailBasisVectorCreationFactor
  res = np.zeros(2)
  res[0] = 0
  res[1] = 0
  
  while (abs(coolIndex-coolIndex2) < minNbPointsBetweenTwoBordersIndexes) and (k < 20):

    for i in range(0, len(bodyContour)):
      Pt   = bodyContour[i][0]
      dist = (Pt[0] - aaa) * (Pt[0] - aaa) + (Pt[1] - bbb) * (Pt[1]-bbb)
      if (dist < min):
        coolIndex = i
        min       = dist
		
    res[0] = coolIndex
    Pt = bodyContour[coolIndex][0]
		
    z = (k+1) * aaa - k * (Pt[0])
    v = (k+1) * bbb - k * (Pt[1])
    min = 10000000
    
    for i in range(0, len(bodyContour)):
      Pt   = bodyContour[i][0]
      dist = (Pt[0] - z) * (Pt[0] - z) + (Pt[1] - v) * (Pt[1] - v);
      if (dist < min):
        coolIndex2 = i
        min        = dist
    res[1] = coolIndex2
    k = k + 1
  
  pt1 = bodyContour[int(res[0])][0]
  pt2 = bodyContour[int(res[1])][0]
  
  if False:
    cv2.circle(dst, (pt1[0],pt1[1]), 1, (0, 0, 255), -1)
    cv2.circle(dst, (pt2[0],pt2[1]), 1, (0, 0, 255), -1)
    cv2.imshow('Frame', dst)
    cv2.waitKey(0)
  
  return res



import tkinter as tk
from tkinter import Tk, mainloop, TOP
from threading import Timer
import cv2
import cvui
import numpy as np

from vars import getGlobalVariables
globalVariables = getGlobalVariables()

def launchStuff():
  if globalVariables["mac"] != 1 and globalVariables["lin"] != 1:
    WINDOW_NAME = "Tracking in Progress"
    cvui.init(WINDOW_NAME)
    cv2.moveWindow(WINDOW_NAME, 0, 0)
    f = open("trace.txt", "r")
    if f.mode == 'r':
      contents = f.read()
    f.close()
    while not("ZebraZoom Analysis finished for" in contents):
      f = open("trace.txt", "r")
      if f.mode == 'r':
        contents = f.read()
      f.close()
      frameCtrl = np.full((400, 500), 100).astype('uint8')
      contentList = contents.split("\n")
      for idx, txt in enumerate(contentList):
        cvui.text(frameCtrl, 4, 15 + idx * 20, txt)
      cvui.imshow(WINDOW_NAME, frameCtrl)
      cv2.waitKey(20)
    cv2.destroyAllWindows()

def initialise():
  if globalVariables["mac"] != 1 and globalVariables["lin"] != 1:
    f = open("trace.txt","w+")
    f.write("")
    f.close()
    f = Timer(0.5, launchStuff, ())
    f.start()

def prepend(text):
  if globalVariables["mac"] != 1 and globalVariables["lin"] != 1:
    with open("trace.txt", "r+") as f:
      content = f.read()
      f.seek(0, 0)
      f.write(text.rstrip('\r\n') + '\n' + content)
